from typing import Any, List, Literal, Optional, Union

import httpx

from xima_ai._vendor.openai._types import NotGiven, Omit, not_given, omit
from xima_ai._vendor.openai.resources.realtime.realtime import (
    AsyncRealtimeConnectionManager,
    RealtimeConnectionManager,
)
from xima_ai._vendor.openai.types.realtime.realtime_audio_config_param import (
    RealtimeAudioConfigParam,
)
from xima_ai._vendor.openai.types.realtime.realtime_session_create_request_param import (
    RealtimeSessionCreateRequestParam,
)
from xima_ai._vendor.openai.types.realtime.realtime_tool_choice_config_param import (
    RealtimeToolChoiceConfigParam,
)
from xima_ai._vendor.openai.types.realtime.realtime_tools_config_param import (
    RealtimeToolsConfigParam,
)
from xima_ai._vendor.openai.types.realtime.realtime_tracing_config_param import (
    RealtimeTracingConfigParam,
)
from xima_ai._vendor.openai.types.realtime.realtime_truncation_param import (
    RealtimeTruncationParam,
)
from xima_ai._vendor.openai.types.responses.response_prompt_param import (
    ResponsePromptParam,
)
from xima_ai._vendor.openai.types.websocket_connection_options import (
    WebsocketConnectionOptions,
)
from xima_ai.api_resources.apis.api_resource import APIResource, AsyncAPIResource
from xima_ai.api_resources.client import AsyncPortkey, XimaAi
from xima_ai.api_resources.types.shared_types import Body, Headers, Query


class MainRealtime(APIResource):
    def __init__(self, client: XimaAi) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client
        self.client_secrets = ClientSecrets(client)
        self.calls = Calls(client)

    def connect(
        self,
        *,
        model: str,
        extra_query: Query = {},
        extra_headers: Headers = {},
        websocket_connection_options: WebsocketConnectionOptions = {},
    ) -> RealtimeConnectionManager:
        return self.openai_client.realtime.connect(
            model=model,
            extra_query=extra_query,
            extra_headers=extra_headers,
            websocket_connection_options=websocket_connection_options,
        )


class AsyncMainRealtime(AsyncAPIResource):
    def __init__(self, client: AsyncPortkey) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client
        self.client_secrets = AsyncClientSecrets(client)
        self.calls = AsyncCalls(client)

    def connect(
        self,
        *,
        model: str,
        extra_query: Query = {},
        extra_headers: Headers = {},
        websocket_connection_options: WebsocketConnectionOptions = {},
    ) -> AsyncRealtimeConnectionManager:
        return self.openai_client.realtime.connect(
            model=model,
            extra_query=extra_query,
            extra_headers=extra_headers,
            websocket_connection_options=websocket_connection_options,
        )


class ClientSecrets(APIResource):
    def __init__(self, client: XimaAi) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    def create(
        self,
        *,
        expires_after: Any,
        session: Any,
        **kwargs: Any,
    ) -> Any:
        extra_headers = kwargs.pop("extra_headers", None)
        extra_query = kwargs.pop("extra_query", None)
        extra_body = kwargs.pop("extra_body", None)
        timeout = kwargs.pop("timeout", None)

        return self.openai_client.realtime.client_secrets.create(
            expires_after=expires_after,
            session=session,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body={**(extra_body or {}), **kwargs},
            timeout=timeout,
        )


class AsyncClientSecrets(AsyncAPIResource):
    def __init__(self, client: AsyncPortkey) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    async def create(
        self,
        *,
        expires_after: Any,
        session: Any,
        **kwargs: Any,
    ) -> Any:
        extra_headers = kwargs.pop("extra_headers", None)
        extra_query = kwargs.pop("extra_query", None)
        extra_body = kwargs.pop("extra_body", None)
        timeout = kwargs.pop("timeout", None)

        return await self.openai_client.realtime.client_secrets.create(
            expires_after=expires_after,
            session=session,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body={**(extra_body or {}), **kwargs},
            timeout=timeout,
        )


class Calls(APIResource):
    def __init__(self, client: XimaAi) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    def create(
        self,
        *,
        sdp: str,
        session: Union[RealtimeSessionCreateRequestParam, Omit] = omit,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> Any:
        return self.openai_client.realtime.calls.create(
            sdp=sdp,
            session=session,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )

    def accept(
        self,
        call_id: str,
        *,
        type: Literal["realtime"],
        audio: Union[RealtimeAudioConfigParam, Omit] = omit,
        include: Union[
            List[Literal["item.input_audio_transcription.logprobs"]], Omit
        ] = omit,
        instructions: Union[str, Omit] = omit,
        max_output_tokens: Union[int, Literal["inf"], Omit] = omit,
        model: Union[
            str,
            Literal[
                "gpt-realtime",
                "gpt-realtime-2025-08-28",
                "gpt-4o-realtime-preview",
                "gpt-4o-realtime-preview-2024-10-01",
                "gpt-4o-realtime-preview-2024-12-17",
                "gpt-4o-realtime-preview-2025-06-03",
                "gpt-4o-mini-realtime-preview",
                "gpt-4o-mini-realtime-preview-2024-12-17",
                "gpt-realtime-mini",
                "gpt-realtime-mini-2025-10-06",
                "gpt-audio-mini",
                "gpt-audio-mini-2025-10-06",
            ],
            Omit,
        ] = omit,
        output_modalities: Union[List[Literal["text", "audio"]], Omit] = omit,
        prompt: Union[Optional[ResponsePromptParam], Omit] = omit,
        tool_choice: Union[RealtimeToolChoiceConfigParam, Omit] = omit,
        tools: Union[RealtimeToolsConfigParam, Omit] = omit,
        tracing: Union[Optional[RealtimeTracingConfigParam], Omit] = omit,
        truncation: Union[RealtimeTruncationParam, Omit] = omit,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> None:
        return self.openai_client.realtime.calls.accept(
            call_id=call_id,
            type=type,
            audio=audio,
            include=include,
            instructions=instructions,
            max_output_tokens=max_output_tokens,
            model=model,
            output_modalities=output_modalities,
            prompt=prompt,
            tool_choice=tool_choice,
            tools=tools,
            tracing=tracing,
            truncation=truncation,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )

    def hangup(
        self,
        call_id: str,
        *,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> None:
        return self.openai_client.realtime.calls.hangup(
            call_id=call_id,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )

    def refer(
        self,
        call_id: str,
        *,
        target_uri: str,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> None:
        return self.openai_client.realtime.calls.refer(
            call_id=call_id,
            target_uri=target_uri,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )

    def reject(
        self,
        call_id: str,
        *,
        status_code: Union[int, Omit] = omit,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> None:
        return self.openai_client.realtime.calls.reject(
            call_id=call_id,
            status_code=status_code,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )


class AsyncCalls(AsyncAPIResource):
    def __init__(self, client: AsyncPortkey) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    async def create(
        self,
        *,
        sdp: str,
        session: Union[RealtimeSessionCreateRequestParam, Omit] = omit,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> Any:
        return await self.openai_client.realtime.calls.create(
            sdp=sdp,
            session=session,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )

    async def accept(
        self,
        call_id: str,
        *,
        type: Literal["realtime"],
        audio: Union[RealtimeAudioConfigParam, Omit] = omit,
        include: Union[
            List[Literal["item.input_audio_transcription.logprobs"]], Omit
        ] = omit,
        instructions: Union[str, Omit] = omit,
        max_output_tokens: Union[int, Literal["inf"], Omit] = omit,
        model: Union[
            str,
            Literal[
                "gpt-realtime",
                "gpt-realtime-2025-08-28",
                "gpt-4o-realtime-preview",
                "gpt-4o-realtime-preview-2024-10-01",
                "gpt-4o-realtime-preview-2024-12-17",
                "gpt-4o-realtime-preview-2025-06-03",
                "gpt-4o-mini-realtime-preview",
                "gpt-4o-mini-realtime-preview-2024-12-17",
                "gpt-realtime-mini",
                "gpt-realtime-mini-2025-10-06",
                "gpt-audio-mini",
                "gpt-audio-mini-2025-10-06",
            ],
            Omit,
        ] = omit,
        output_modalities: Union[List[Literal["text", "audio"]], Omit] = omit,
        prompt: Union[Optional[ResponsePromptParam], Omit] = omit,
        tool_choice: Union[RealtimeToolChoiceConfigParam, Omit] = omit,
        tools: Union[RealtimeToolsConfigParam, Omit] = omit,
        tracing: Union[Optional[RealtimeTracingConfigParam], Omit] = omit,
        truncation: Union[RealtimeTruncationParam, Omit] = omit,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> None:
        return await self.openai_client.realtime.calls.accept(
            call_id=call_id,
            type=type,
            audio=audio,
            include=include,
            instructions=instructions,
            max_output_tokens=max_output_tokens,
            model=model,
            output_modalities=output_modalities,
            prompt=prompt,
            tool_choice=tool_choice,
            tools=tools,
            tracing=tracing,
            truncation=truncation,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )

    async def hangup(
        self,
        call_id: str,
        *,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> None:
        return await self.openai_client.realtime.calls.hangup(
            call_id=call_id,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )

    async def refer(
        self,
        call_id: str,
        *,
        target_uri: str,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> None:
        return await self.openai_client.realtime.calls.refer(
            call_id=call_id,
            target_uri=target_uri,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )

    async def reject(
        self,
        call_id: str,
        *,
        status_code: Union[int, Omit] = omit,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> None:
        return await self.openai_client.realtime.calls.reject(
            call_id=call_id,
            status_code=status_code,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )



from typing import Any, Iterable, List, Union

from xima_ai._vendor.openai.resources.beta.realtime.realtime import (
    AsyncRealtimeConnectionManager,
    RealtimeConnectionManager,
)
from xima_ai._vendor.openai.types.beta.realtime.transcription_session import (
    TranscriptionSession,
)
from xima_ai._vendor.openai.types.websocket_connection_options import (
    WebsocketConnectionOptions,
)
from xima_ai.api_resources.apis.api_resource import APIResource, AsyncAPIResource
from xima_ai.api_resources.client import AsyncPortkey, XimaAi
from xima_ai.api_resources.types.beta_realtime import SessionCreateResponse

from ..._vendor.openai._types import NOT_GIVEN, NotGiven


class BetaRealtime(APIResource):
    def __init__(self, client: XimaAi) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client
        self.sessions = BetaSessions(client)
        self.transcription_sessions = BetaTranscriptionSessions(client)

    def connect(
        self,
        *,
        model: str,
        websocket_connection_options: WebsocketConnectionOptions = {},
        **kwargs,
    ) -> RealtimeConnectionManager:
        return self.openai_client.beta.realtime.connect(
            model=model,
            websocket_connection_options=websocket_connection_options,
            extra_headers=self.openai_client.default_headers,
            **kwargs,
        )


class BetaTranscriptionSessions(APIResource):
    def __init__(self, client: XimaAi) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    def create(
        self,
        *,
        client_secret: Union[Any, NotGiven] = NOT_GIVEN,
        include: Union[List[Any], NotGiven] = NOT_GIVEN,
        input_audio_format: Union[Any, NotGiven] = NOT_GIVEN,
        input_audio_noise_reduction: Union[Any, NotGiven] = NOT_GIVEN,
        input_audio_transcription: Union[Any, NotGiven] = NOT_GIVEN,
        modalities: Union[List[Any], NotGiven] = NOT_GIVEN,
        turn_detection: Union[Any, NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> TranscriptionSession:
        extra_headers = kwargs.pop("extra_headers", None)
        extra_query = kwargs.pop("extra_query", None)
        extra_body = kwargs.pop("extra_body", None)
        timeout = kwargs.pop("timeout", None)
        response = self.openai_client.beta.realtime.transcription_sessions.create(
            client_secret=client_secret,
            include=include,
            input_audio_format=input_audio_format,
            input_audio_noise_reduction=input_audio_noise_reduction,
            input_audio_transcription=input_audio_transcription,
            modalities=modalities,
            turn_detection=turn_detection,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body={**(extra_body or {}), **kwargs},
            timeout=timeout,
        )
        return response


class AsyncBetaRealtime(AsyncAPIResource):
    def __init__(self, client: AsyncPortkey) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client
        self.sessions = AsyncBetaSessions(client)
        self.transcription_sessions = AsyncBetaTranscriptionSessions(client)

    def connect(
        self,
        *,
        model: str,
        websocket_connection_options: WebsocketConnectionOptions = {},
        **kwargs,
    ) -> AsyncRealtimeConnectionManager:
        return self.openai_client.beta.realtime.connect(
            model=model,
            websocket_connection_options=websocket_connection_options,
            extra_headers=self.openai_client.default_headers,
            **kwargs,
        )


class AsyncBetaTranscriptionSessions(AsyncAPIResource):
    def __init__(self, client: AsyncPortkey) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    async def create(
        self,
        *,
        client_secret: Union[Any, NotGiven] = NOT_GIVEN,
        include: Union[List[Any], NotGiven] = NOT_GIVEN,
        input_audio_format: Union[Any, NotGiven] = NOT_GIVEN,
        input_audio_noise_reduction: Union[Any, NotGiven] = NOT_GIVEN,
        input_audio_transcription: Union[Any, NotGiven] = NOT_GIVEN,
        modalities: Union[List[Any], NotGiven] = NOT_GIVEN,
        turn_detection: Union[Any, NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> TranscriptionSession:
        extra_headers = kwargs.pop("extra_headers", None)
        extra_query = kwargs.pop("extra_query", None)
        extra_body = kwargs.pop("extra_body", None)
        timeout = kwargs.pop("timeout", None)
        response = await self.openai_client.beta.realtime.transcription_sessions.create(
            client_secret=client_secret,
            include=include,
            input_audio_format=input_audio_format,
            input_audio_noise_reduction=input_audio_noise_reduction,
            input_audio_transcription=input_audio_transcription,
            modalities=modalities,
            turn_detection=turn_detection,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body={**(extra_body or {}), **kwargs},
            timeout=timeout,
        )
        return response


class BetaSessions(APIResource):
    def __init__(self, client: XimaAi) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    def create(
        self,
        *,
        model: Any = "portkey-default",
        input_audio_format: Union[Any, NotGiven] = NOT_GIVEN,
        input_audio_transcription: Union[Any, NotGiven] = NOT_GIVEN,
        instructions: Union[str, NotGiven] = NOT_GIVEN,
        max_response_output_tokens: Union[int, Any, NotGiven] = NOT_GIVEN,
        modalities: Union[List[Any], NotGiven] = NOT_GIVEN,
        output_audio_format: Union[Any, NotGiven] = NOT_GIVEN,
        temperature: Union[float, NotGiven] = NOT_GIVEN,
        tool_choice: Union[str, NotGiven] = NOT_GIVEN,
        tools: Union[Iterable[Any], NotGiven] = NOT_GIVEN,
        turn_detection: Union[Any, NotGiven] = NOT_GIVEN,
        voice: Union[Any, NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> SessionCreateResponse:
        extra_headers = kwargs.pop("extra_headers", None)
        extra_query = kwargs.pop("extra_query", None)
        extra_body = kwargs.pop("extra_body", None)
        timeout = kwargs.pop("timeout", None)
        response = self.openai_client.beta.realtime.sessions.create(
            model=model,
            input_audio_format=input_audio_format,
            input_audio_transcription=input_audio_transcription,
            instructions=instructions,
            max_response_output_tokens=max_response_output_tokens,
            modalities=modalities,
            output_audio_format=output_audio_format,
            temperature=temperature,
            tool_choice=tool_choice,
            tools=tools,
            turn_detection=turn_detection,
            voice=voice,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body={**(extra_body or {}), **kwargs},
            timeout=timeout,
        )

        return response  # type: ignore[return-value]


class AsyncBetaSessions(AsyncAPIResource):
    def __init__(self, client: AsyncPortkey) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    async def create(
        self,
        *,
        model: Any = "portkey-default",
        input_audio_format: Union[Any, NotGiven] = NOT_GIVEN,
        input_audio_transcription: Union[Any, NotGiven] = NOT_GIVEN,
        instructions: Union[str, NotGiven] = NOT_GIVEN,
        max_response_output_tokens: Union[int, Any, NotGiven] = NOT_GIVEN,
        modalities: Union[List[Any], NotGiven] = NOT_GIVEN,
        output_audio_format: Union[Any, NotGiven] = NOT_GIVEN,
        temperature: Union[float, NotGiven] = NOT_GIVEN,
        tool_choice: Union[str, NotGiven] = NOT_GIVEN,
        tools: Union[Iterable[Any], NotGiven] = NOT_GIVEN,
        turn_detection: Union[Any, NotGiven] = NOT_GIVEN,
        voice: Union[Any, NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> SessionCreateResponse:
        extra_headers = kwargs.pop("extra_headers", None)
        extra_query = kwargs.pop("extra_query", None)
        extra_body = kwargs.pop("extra_body", None)
        timeout = kwargs.pop("timeout", None)
        response = await self.openai_client.beta.realtime.sessions.create(
            model=model,
            input_audio_format=input_audio_format,
            input_audio_transcription=input_audio_transcription,
            instructions=instructions,
            max_response_output_tokens=max_response_output_tokens,
            modalities=modalities,
            output_audio_format=output_audio_format,
            temperature=temperature,
            tool_choice=tool_choice,
            tools=tools,
            turn_detection=turn_detection,
            voice=voice,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body={**(extra_body or {}), **kwargs},
            timeout=timeout,
        )

        return response  # type: ignore[return-value]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["RealtimeResponseUsageInputTokenDetails", "CachedTokensDetails"]


class CachedTokensDetails(BaseModel):
    audio_tokens: Optional[int] = None
    """The number of cached audio tokens used as input for the Response."""

    image_tokens: Optional[int] = None
    """The number of cached image tokens used as input for the Response."""

    text_tokens: Optional[int] = None
    """The number of cached text tokens used as input for the Response."""


class RealtimeResponseUsageInputTokenDetails(BaseModel):
    audio_tokens: Optional[int] = None
    """The number of audio tokens used as input for the Response."""

    cached_tokens: Optional[int] = None
    """The number of cached tokens used as input for the Response."""

    cached_tokens_details: Optional[CachedTokensDetails] = None
    """Details about the cached tokens used as input for the Response."""

    image_tokens: Optional[int] = None
    """The number of image tokens used as input for the Response."""

    text_tokens: Optional[int] = None
    """The number of text tokens used as input for the Response."""



from __future__ import annotations

import json
import os
from enum import Enum, EnumMeta
from typing import Any, Dict, List, Literal, Mapping, Optional, TypeVar, Union, cast

import httpx
from pydantic import BaseModel
from typing_extensions import NotRequired, TypedDict

import xima_ai
from xima_ai.api_resources.types.chat_complete_type import (
    ChatCompletionChunk,
    ChatCompletions,
)
from xima_ai.api_resources.types.complete_type import (
    TextCompletion,
    TextCompletionChunk,
)
from xima_ai.api_resources.types.feedback_type import FeedbackResponse
from xima_ai.api_resources.types.generation_type import (
    PromptCompletion,
    PromptCompletionChunk,
    PromptRender,
)

from .._vendor.openai._types import NOT_GIVEN
from .exceptions import (
    APIStatusError,
    AuthenticationError,
    BadRequestError,
    ConflictError,
    InternalServerError,
    NotFoundError,
    PermissionDeniedError,
    RateLimitError,
    UnprocessableEntityError,
)
from .global_constants import (
    LOCAL_BASE_URL,
    MISSING_API_KEY_ERROR_MESSAGE,
    MISSING_BASE_URL,
    MISSING_MODE_MESSAGE,
    XIMAAI_API_KEY_ENV,
    XIMAAI_BASE_URL,
    XIMAAI_HEADER_PREFIX,
    XIMAAI_PROXY_ENV,
)


class MetaEnum(EnumMeta):
    def __contains__(cls, item):
        try:
            cls(item)
        except ValueError:
            return False
        return True


class CacheType(str, Enum, metaclass=MetaEnum):
    SEMANTIC = "semantic"
    SIMPLE = "simple"


CacheLiteral = Literal["semantic", "simple"]


ResponseT = TypeVar(
    "ResponseT",
    bound="Union[ChatCompletionChunk, ChatCompletions, TextCompletion, TextCompletionChunk, GenericResponse, PromptCompletion, PromptCompletionChunk, PromptRender, FeedbackResponse, httpx.Response]",  # noqa: E501
)


class ProviderTypes(str, Enum):
    """_summary_

    Args:
        Enum (_type_): _description_

    Returns:
        _type_: _description_
    """

    OPENAI = "openai"
    COHERE = "cohere"
    ANTHROPIC = "anthropic"
    AZURE_OPENAI = "azure-openai"
    HUGGING_FACE = "huggingface"


ProviderTypesLiteral = Literal[
    "openai", "cohere", "anthropic", "azure-openai", "huggingface"
]


class Modes(str, Enum, metaclass=MetaEnum):
    """_summary_

    Args:
        Enum (_type_): _description_
    """

    FALLBACK = "fallback"
    AB_TEST = "ab_test"
    SINGLE = "single"
    PROXY = "proxy"


class ApiType(str, Enum, metaclass=MetaEnum):
    COMPLETIONS = "completions"
    CHAT_COMPLETION = "chat_completions"


ModesLiteral = Literal["fallback", "ab_test", "single", "proxy"]


class XimaAiApiPaths(str, Enum, metaclass=MetaEnum):
    GENERATION = "/prompts/{prompt_id}/generate"
    CHAT_COMPLETE_API = "/chat/completions"
    TEXT_COMPLETE_API = "/completions"
    PROMPT_API = "/prompt/complete"
    FEEDBACK_API = "/feedback"
    EMBEDDING_API = "/embeddings"
    USER_API = "/admin/users"
    INVITE_API = "/admin/users/invites"
    WORKSPACE_API = "/admin/workspaces"
    CONFIG_API = "/configs"
    API_KEYS_API = "/api-keys"
    VIRTUAL_KEYS_API = "/virtual-keys"
    LOGS_EXPORT_API = "/logs/exports"
    LOGS_API = "/logs"
    LABELS_API = "/labels"
    COLLECTIONS_API = "/collections"
    PROMPTS_API = "/prompts"
    PROMPTS_PARTIALS_API = "/prompts/partials"
    INTEGRATIONS_API = "/integrations"
    PROVIDERS_API = "/providers"
    GUARDRAILS_API = "/guardrails"

    def __str__(self):
        return self.value


class Options(BaseModel):
    method: str
    url: str
    params: Optional[Mapping[str, str]] = None
    headers: Optional[Mapping[str, str]] = None
    max_retries: Optional[int] = None
    timeout: Optional[float] = None
    # stringified json
    data: Optional[Mapping[str, Any]] = None
    # json structure
    json_body: Optional[Mapping[str, Any]] = None
    files: Any = None


class FunctionCall(BaseModel):
    name: str
    arguments: str


class ToolCall(BaseModel):
    id: str
    function: FunctionCall
    type: str


class DeltaToolCallFunction(BaseModel):
    arguments: Optional[str] = None
    name: Optional[str] = None


class DeltaToolCall(BaseModel):
    index: int
    id: Optional[str] = None
    function: Optional[DeltaToolCallFunction] = None
    type: Optional[str] = None


class Message(TypedDict):
    role: str
    content: NotRequired[Union[None, str]]
    tool_calls: NotRequired[Union[None, List[ToolCall]]]


class Function(BaseModel):
    name: str
    description: str
    parameters: Dict[str, object]


class Tool(BaseModel):
    function: Function
    type: str


class RetrySettings(TypedDict):
    attempts: int
    on_status_codes: list


class ConversationInput(BaseModel):
    prompt: Optional[str] = None
    messages: Optional[List[Message]] = None


class ModelParams(BaseModel):
    model: Optional[str] = None
    suffix: Optional[str] = None
    max_tokens: Optional[int] = None
    temperature: Optional[float] = None
    top_k: Optional[int] = None
    top_p: Optional[float] = None
    n: Optional[int] = None
    stop_sequences: Optional[List[str]] = None
    timeout: Union[float, None] = None
    functions: Optional[List[Function]] = None
    function_call: Optional[Union[None, str, Function]] = None
    logprobs: Optional[bool] = None
    top_logprobs: Optional[int] = None
    echo: Optional[bool] = None
    stop: Optional[Union[str, List[str]]] = None
    presence_penalty: Optional[int] = None
    frequency_penalty: Optional[int] = None
    best_of: Optional[int] = None
    logit_bias: Optional[Dict[str, int]] = None
    user: Optional[str] = None
    organization: Optional[str] = None
    tool_choice: Optional[Union[None, str]] = None
    tools: Optional[List[Tool]] = None


class OverrideParams(ModelParams, ConversationInput): ...


def remove_empty_values(
    data: Union[Dict[str, Any], Mapping[str, Any]],
) -> Dict[str, Any]:
    if isinstance(data, dict):
        cleaned_dict = {}
        for key, value in data.items():
            if value is not NOT_GIVEN and value != "":
                cleaned_value = remove_empty_values(value)
                if cleaned_value is not NOT_GIVEN and cleaned_value != "":
                    cleaned_dict[key] = cleaned_value
        return cleaned_dict
    elif isinstance(data, list):
        cleaned_list = []

        for item in data:  # type: ignore
            cleaned_item = remove_empty_values(item)
            if cleaned_item is not NOT_GIVEN and cleaned_item != "":
                cleaned_list.append(cleaned_item)
        return cleaned_list  # type: ignore
    else:
        return cast(dict, data)


class Constructs(BaseModel):
    provider: Union[ProviderTypes, ProviderTypesLiteral, str]
    api_key: Optional[str] = None
    virtual_key: Optional[str] = None
    cache: Optional[bool] = None
    cache_age: Optional[int] = None
    cache_status: Optional[Union[CacheType, CacheLiteral]] = None
    cache_force_refresh: Optional[bool] = None
    trace_id: Optional[str] = None
    metadata: Optional[Dict[str, Any]] = None
    debug: Optional[bool] = None
    custom_host: Optional[str] = None
    forward_headers: Optional[str] = None
    instrumentation: Optional[bool] = None
    weight: Optional[float] = None
    retry: Optional[RetrySettings] = None
    deployment_id: Optional[str] = None
    resource_name: Optional[str] = None
    api_version: Optional[str] = None
    openai_project: Optional[str] = None
    openai_organization: Optional[str] = None
    aws_secret_access_key: Optional[str] = None
    aws_access_key_id: Optional[str] = None
    aws_session_token: Optional[str] = None
    aws_region: Optional[str] = None
    vertex_project_id: Optional[str] = None
    vertex_region: Optional[str] = None
    workers_ai_account_id: Optional[str] = None
    azure_resource_name: Optional[str] = None
    azure_deployment_id: Optional[str] = None
    azure_api_version: Optional[str] = None
    azure_endpoint_name: Optional[str] = None
    huggingface_base_url: Optional[str] = None
    cache_namespace: Optional[str] = None
    request_timeout: Optional[int] = None
    anthropic_beta: Optional[str] = None
    anthropic_version: Optional[str] = None
    mistral_fim_completion: Optional[bool] = None
    vertex_storage_bucket_name: Optional[str] = None
    provider_file_name: Optional[str] = None
    provider_model: Optional[str] = None
    aws_s3_bucket: Optional[str] = None
    aws_s3_object_key: Optional[str] = None
    aws_bedrock_model: Optional[str] = None
    fireworks_account_id: Optional[str] = None


class LLMOptions(Constructs, ConversationInput, ModelParams):
    @classmethod
    def parse_api_key(cls, api_key, values):
        if api_key is None and values.get("virtual_key") is None:
            # You can access other fields' values via the 'values' dictionary
            provider = values.get("provider", "")
            api_key = apikey_from_env(provider)
        return api_key


class ProviderOptions(Constructs):
    override_params: Optional[OverrideParams] = None

    @classmethod
    def parse_cache_age(cls, cache_age):
        if cache_age is not None:
            cache_age = f"max-age={cache_age}"
        return cache_age


class RequestConfig(BaseModel):
    mode: str
    options: List[ProviderOptions]


class Body(LLMOptions): ...


class ConfigSlug(BaseModel):
    config: str


class Params(Constructs, ConversationInput, ModelParams, extra="forbid"): ...


class RequestData(BaseModel):
    config: RequestConfig
    params: Params


class XimaAiResponse(BaseModel):
    model: str
    choices: List[Any]
    raw_body: Dict[str, Any]

    def __str__(self):
        return json.dumps(self.dict(), indent=4)


class Usage(BaseModel, extra="allow"):
    prompt_tokens: Optional[int] = None
    completion_tokens: Optional[int] = None
    total_tokens: Optional[int] = None

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class GenericResponse(BaseModel, extra="allow"):
    success: Optional[bool] = None
    data: Optional[Any] = None
    warning: Optional[str] = None
    _headers: Optional[httpx.Headers] = None

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers_generic(self._headers)


def apikey_from_env(provider: Union[ProviderTypes, ProviderTypesLiteral, str]) -> str:
    env_key = f"{provider.upper().replace('-', '_')}_API_KEY"
    if provider is None:
        return ""
    if env_key in os.environ and os.environ[env_key]:
        return os.environ.get(env_key, "")

    raise ValueError(
        f"Did not find '{provider.lower()}' api key, please add an environment variable"
        f" `{env_key}` which contains it, or pass"
        f"  `api_key` as a named parameter in LLMOptions"
    )


def make_status_error(
    err_msg: str,
    *,
    body: object,
    request: httpx.Request,
    response: httpx.Response,
) -> APIStatusError:
    if response.status_code == 400:
        return BadRequestError(err_msg, request=request, response=response, body=body)
    if response.status_code == 401:
        return AuthenticationError(
            err_msg, request=request, response=response, body=body
        )
    if response.status_code == 403:
        return PermissionDeniedError(
            err_msg, request=request, response=response, body=body
        )
    if response.status_code == 404:
        return NotFoundError(err_msg, request=request, response=response, body=body)
    if response.status_code == 409:
        return ConflictError(err_msg, request=request, response=response, body=body)
    if response.status_code == 422:
        return UnprocessableEntityError(
            err_msg, request=request, response=response, body=body
        )
    if response.status_code == 429:
        return RateLimitError(err_msg, request=request, response=response, body=body)
    if response.status_code >= 500:
        return InternalServerError(
            err_msg, request=request, response=response, body=body
        )
    return APIStatusError(err_msg, request=request, response=response, body=body)


class Config(BaseModel):
    api_key: Optional[str] = None
    base_url: Optional[str] = None
    mode: Optional[Union[Modes, ModesLiteral, str]] = None
    llms: Optional[Union[List[LLMOptions], LLMOptions]] = None

    @classmethod
    def check_mode(cls, mode):
        if mode is None:
            # You can access other fields' values via the 'values' dictionary
            mode = retrieve_mode()

        return mode

    @classmethod
    def parse_llms(cls, llms):
        if isinstance(llms, LLMOptions):
            llms = [llms]
        return llms


def default_api_key(base_url, api_key) -> str:
    if api_key:
        return api_key
    env_api_key = os.environ.get(XIMAAI_API_KEY_ENV, "")
    if base_url == XIMAAI_BASE_URL:
        if env_api_key:
            return env_api_key
        raise ValueError(MISSING_API_KEY_ERROR_MESSAGE)
    else:
        return env_api_key


def default_base_url() -> str:
    if xima_ai.base_url:
        print("xxxxxxxxxxxxxxxxxxxxxxxxxxxxxx", xima_ai.base_url)
        return xima_ai.base_url

    env_base_url = os.environ.get(XIMAAI_PROXY_ENV, XIMAAI_BASE_URL)
    if env_base_url:
        return env_base_url
    raise ValueError(MISSING_BASE_URL)


def retrieve_config() -> Union[Mapping, str]:
    if xima_ai.config:
        return xima_ai.config
    # raise ValueError(MISSING_CONFIG_MESSAGE)
    return {}


def retrieve_mode() -> Union[Modes, ModesLiteral, str]:
    if xima_ai.mode:
        return xima_ai.mode
    raise ValueError(MISSING_MODE_MESSAGE)


def get_portkey_header(key: str) -> str:
    return f"{XIMAAI_HEADER_PREFIX}{key}"


def parse_headers_generic(headers: Optional[httpx.Headers]) -> dict:
    if headers is None:
        return {}

    _headers = {}
    for k, v in headers.items():
        if k.startswith(XIMAAI_HEADER_PREFIX):
            k = k.replace(XIMAAI_HEADER_PREFIX, "")
            _headers[k] = v

    return _headers


def set_base_url(base_url, api_key):
    if base_url:
        return base_url

    env_base_url = os.environ.get(XIMAAI_BASE_URL) or os.environ.get(XIMAAI_PROXY_ENV)

    if env_base_url:
        return env_base_url
    api_key = api_key or os.environ.get(XIMAAI_API_KEY_ENV)
    return XIMAAI_BASE_URL if api_key else LOCAL_BASE_URL


def create_model_instance(model_class):
    """
    Create a model instance that's compatible with both Pydantic v1 and v2.

    This function provides backward compatibility for the transition from Pydantic v1
    to Pydantic v2. In Pydantic v1, the method was called `construct()`, while in
    Pydantic v2, it was renamed to `model_construct()`.
    """
    try:
        # Try Pydantic v2 method first
        return model_class.model_construct()
    except AttributeError:
        # Fall back to Pydantic v1 method
        return model_class.construct()



from __future__ import annotations
import inspect

import os
from os import walk
from typing import Any, Dict, List
import pytest
from uuid import uuid4
from portkey_ai import AsyncPortkey
from time import sleep
from dotenv import load_dotenv
from .utils import read_json_file


load_dotenv(override=True)

base_url = os.environ.get("PORTKEY_BASE_URL")
api_key = os.environ.get("PORTKEY_API_KEY")
virtual_api_key = os.environ.get("OPENAI_VIRTUAL_KEY")

CONFIGS_PATH = "./tests/configs/moderations"


def get_configs(folder_path) -> List[Dict[str, Any]]:
    config_files = []
    for dirpath, _, file_names in walk(folder_path):
        for f in file_names:
            config_files.append(read_json_file(os.path.join(dirpath, f)))

    return config_files


class TestModerations:
    client = AsyncPortkey
    parametrize = pytest.mark.parametrize("client", [client], ids=["strict"])
    models = read_json_file("./tests/models.json")

    def get_metadata(self):
        return {
            "case": "testing",
            "function": inspect.currentframe().f_back.f_code.co_name,
            "random_id": str(uuid4()),
        }

    # --------------------------
    # Test-1
    t1_params = []
    t = []
    for k, v in models.items():
        if k == "openai":
            for i in v["chat"]:
                t.append((client, k, os.environ.get(v["env_variable"]), i))

        t1_params.extend(t)

    @pytest.mark.asyncio
    @pytest.mark.parametrize("client, provider, auth, model", t1_params)
    async def test_method_single_with_vk_and_provider(
        self, client: Any, provider: str, auth: str, model
    ) -> None:
        portkey = client(
            base_url=base_url,
            api_key=api_key,
            provider=f"{provider}",
            Authorization=f"{auth}",
            trace_id=str(uuid4()),
            metadata=self.get_metadata(),
        )

        moderations = await portkey.moderations.create(
            input="I want to kill them.", model="text-moderation-stable"
        )

        assert isinstance(moderations.id, str) is True

    # --------------------------
    # Test -2
    t2_params = []
    for i in get_configs(f"{CONFIGS_PATH}/single_with_basic_config"):
        t2_params.append((client, i))

    @pytest.mark.asyncio
    @pytest.mark.parametrize("client, config", t2_params)
    async def test_method_single_with_basic_config(
        self, client: Any, config: Dict
    ) -> None:
        portkey = client(
            base_url=base_url,
            api_key=api_key,
            trace_id=str(uuid4()),
            metadata=self.get_metadata(),
            config=config,
        )

        moderations = await portkey.moderations.create(
            input="I want to kill them.", model="text-moderation-stable"
        )

        assert isinstance(moderations.id, str) is True

    # --------------------------
    # Test-3
    t3_params = []
    for i in get_configs(f"{CONFIGS_PATH}/single_provider_with_vk_retry_cache"):
        t3_params.append((client, i))

    @pytest.mark.asyncio
    @pytest.mark.parametrize("client, config", t3_params)
    async def test_method_single_provider_with_vk_retry_cache(
        self, client: Any, config: Dict
    ) -> None:
        # 1. Make a new cache the cache
        # 2. Make a cache hit and see if the response contains the data.
        random_id = str(uuid4())
        metadata = self.get_metadata()
        portkey = client(
            base_url=base_url,
            api_key=api_key,
            trace_id=random_id,
            virtual_key=virtual_api_key,
            metadata=metadata,
            config=config,
        )

        moderations = await portkey.moderations.create(
            input="I want to kill them.", model="text-moderation-stable"
        )

        assert isinstance(moderations.id, str) is True
        # Sleeping for the cache to reflect across the workers. The cache has an
        # eventual consistency and not immediate consistency.
        sleep(20)
        portkey_2 = client(
            base_url=base_url,
            api_key=api_key,
            trace_id=random_id,
            virtual_key=virtual_api_key,
            metadata=metadata,
            config=config,
        )

        cached_moderations = await portkey_2.moderations.create(
            input="I want to kill them.", model="text-moderation-stable"
        )

        assert isinstance(cached_moderations.id, str) is True



from __future__ import annotations

import os
from typing import Any

import pytest

from tests.utils import read_json_file
from portkey_ai.langchain import LangchainCallbackHandler
from langchain.chat_models import ChatOpenAI
from langchain_core.prompts import ChatPromptTemplate
from langchain.chains import LLMChain

api_key = os.environ.get("PORTKEY_API_KEY")


class TestLLMLangchain:
    client = LangchainCallbackHandler
    parametrize = pytest.mark.parametrize("client", [client], ids=["strict"])
    models = read_json_file("./tests/models.json")

    t1_params = []
    t1 = []
    for k, v in models.items():
        if k == "langchain":
            for i in v["chat"]:
                t1.append((client, i))

            t1_params.extend(t1)

    @pytest.mark.parametrize("client, model", t1_params)
    def test_method_langchain_openai(self, client: Any, model) -> None:
        handler = client(
            api_key=api_key,
        )
        llm = ChatOpenAI(callbacks=[handler], model=model)  # type: ignore[misc]
        prompt = ChatPromptTemplate.from_messages(
            [
                ("system", "You are world class technical documentation writer."),
                ("user", "{input}"),
            ]
        )
        chain = LLMChain(llm=llm, prompt=prompt)

        assert (
            isinstance(chain.invoke({"input": "what is langchain?"}).get("input"), str)
            is True
        )
        assert (
            isinstance(chain.invoke({"input": "what is langchain?"}).get("text"), str)
            is True
        )



import json
from typing import Any, Dict, List, Optional
import httpx
from pydantic import BaseModel, PrivateAttr
from .utils import parse_headers

__all__ = [
    "FileCreateResponse",
    "FileRetrieveResponse",
    "FileListResponse",
]


class FileCreateResponse(BaseModel, extra="allow"):
    id: Optional[str] = None
    bytes: Optional[int] = None
    container_id: Optional[str] = None
    created_at: Optional[int] = None
    object: Optional[str] = None
    path: Optional[str] = None
    source: Optional[str] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class FileRetrieveResponse(BaseModel, extra="allow"):
    id: Optional[str] = None
    bytes: Optional[int] = None
    container_id: Optional[str] = None
    created_at: Optional[int] = None
    object: Optional[str] = None
    path: Optional[str] = None
    source: Optional[str] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class FileListResponse(BaseModel, extra="allow"):
    object: Optional[str] = None
    data: Optional[List[FileRetrieveResponse]] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)



from typing import Union

from xima_ai._vendor.openai._types import HeadersLike
from xima_ai._vendor.openai.types.webhooks.unwrap_webhook_event import (
    UnwrapWebhookEvent,
)
from xima_ai.api_resources.apis.api_resource import APIResource, AsyncAPIResource
from xima_ai.api_resources.client import AsyncPortkey, XimaAi


class Webhooks(APIResource):
    def __init__(self, client: XimaAi) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    def unwrap(
        self,
        payload: Union[str, bytes],
        headers: HeadersLike,
        *,
        secret: Union[str, None] = None,
    ) -> UnwrapWebhookEvent:
        return self.openai_client.webhooks.unwrap(
            payload=payload,
            headers=headers,
            secret=secret,
        )

    def verify_signature(
        self,
        payload: Union[str, bytes],
        headers: HeadersLike,
        *,
        secret: Union[str, None] = None,
        tolerance: int = 300,
    ) -> None:
        return self.openai_client.webhooks.verify_signature(
            payload=payload,
            headers=headers,
            secret=secret,
            tolerance=tolerance,
        )


class AsyncWebhooks(AsyncAPIResource):
    def __init__(self, client: AsyncPortkey) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    def unwrap(
        self,
        payload: Union[str, bytes],
        headers: HeadersLike,
        *,
        secret: Union[str, None] = None,
    ) -> UnwrapWebhookEvent:
        return self.openai_client.webhooks.unwrap(
            payload=payload,
            headers=headers,
            secret=secret,
        )

    def verify_signature(
        self,
        payload: Union[str, bytes],
        headers: HeadersLike,
        *,
        secret: Union[str, None] = None,
        tolerance: int = 300,
    ) -> None:
        return self.openai_client.webhooks.verify_signature(
            payload=payload,
            headers=headers,
            secret=secret,
            tolerance=tolerance,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["FilePart"]


class FilePart(BaseModel):
    id: str
    """Unique identifier for the uploaded file."""

    mime_type: Optional[str] = None
    """MIME type reported for the uploaded file. Defaults to null when unknown."""

    name: Optional[str] = None
    """Original filename supplied by the uploader. Defaults to null when unnamed."""

    type: Literal["file"]
    """Type discriminator that is always `file`."""

    upload_url: Optional[str] = None
    """Signed URL for downloading the uploaded file.

    Defaults to null when no download link is available.
    """



import json
from typing import Any, Dict, List, Optional

import httpx
from pydantic import BaseModel, PrivateAttr

from xima_ai.api_resources.types.utils import parse_headers


class Provider(BaseModel, extra="allow"):
    id: Optional[str] = None
    slug: Optional[str] = None
    name: Optional[str] = None
    note: Optional[str] = None
    integration_id: Optional[str] = None
    workspace_id: Optional[str] = None
    usage_limits: Optional[Dict[str, Any]] = None
    rate_limits: Optional[Dict[str, Any]] = None
    expires_at: Optional[str] = None
    created_at: Optional[str] = None
    last_updated_at: Optional[str] = None
    object: Optional[str] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class ProviderListResponse(BaseModel, extra="allow"):
    object: Optional[str] = None
    total: Optional[int] = None
    data: Optional[List[Provider]] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class ProviderDetailResponse(BaseModel, extra="allow"):
    id: Optional[str] = None
    slug: Optional[str] = None
    name: Optional[str] = None
    note: Optional[str] = None
    integration_id: Optional[str] = None
    workspace_id: Optional[str] = None
    usage_limits: Optional[Dict[str, Any]] = None
    rate_limits: Optional[List[Any]] = None
    expires_at: Optional[str] = None
    created_at: Optional[str] = None
    last_updated_at: Optional[str] = None
    object: Optional[str] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class ProviderCreateResponse(BaseModel, extra="allow"):
    id: Optional[str] = None
    slug: Optional[str] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class ProviderUpdateResponse(BaseModel, extra="allow"):
    id: Optional[str] = None
    slug: Optional[str] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default



import json
from typing import Any, Literal, Optional, Union

import httpx

from xima_ai._vendor.openai.types.beta.chatkit.chat_session_chatkit_configuration_param import (
    ChatSessionChatKitConfigurationParam,
)
from xima_ai._vendor.openai.types.beta.chatkit.chat_session_expires_after_param import (
    ChatSessionExpiresAfterParam,
)
from xima_ai._vendor.openai.types.beta.chatkit.chat_session_rate_limits_param import (
    ChatSessionRateLimitsParam,
)
from xima_ai._vendor.openai.types.beta.chatkit.chat_session_workflow_param import (
    ChatSessionWorkflowParam,
)
from xima_ai._vendor.openai.types.beta.chatkit_upload_file_response import (
    ChatKitUploadFileResponse,
)
from xima_ai.api_resources.apis.api_resource import APIResource, AsyncAPIResource
from xima_ai.api_resources.client import AsyncPortkey, XimaAi
from xima_ai.api_resources.types.chatkit_type import (
    ChatKitThread,
    ChatKitThreadList,
    ChatSession,
    ThreadDeleteResponse,
)
from xima_ai.api_resources.types.shared_types import Body, Headers, Query

from ..._vendor.openai._types import (
    FileTypes,
    NotGiven,
    Omit,
    not_given,
    omit,
)


class ChatKit(APIResource):
    def __init__(self, client: XimaAi) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client
        self.sessions = ChatKitSessions(client)
        self.threads = ChatKitThreads(client)

    def upload_file(
        self,
        *,
        file: FileTypes,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> ChatKitUploadFileResponse:
        response = self.openai_client.beta.chatkit.upload_file(
            file=file,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        return response


class ChatKitSessions(APIResource):
    def __init__(self, client: XimaAi) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    def create(
        self,
        *,
        user: str,
        workflow: ChatSessionWorkflowParam,
        chatkit_configuration: Union[ChatSessionChatKitConfigurationParam, Omit] = omit,
        expires_after: Union[ChatSessionExpiresAfterParam, Omit] = omit,
        rate_limits: Union[ChatSessionRateLimitsParam, Omit] = omit,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> ChatSession:
        response = self.openai_client.with_raw_response.beta.chatkit.sessions.create(
            user=user,
            workflow=workflow,
            chatkit_configuration=chatkit_configuration,
            expires_after=expires_after,
            rate_limits=rate_limits,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        data = ChatSession(**json.loads(response.text))
        data._headers = response.headers

        return data

    def cancel(
        self,
        session_id: str,
        *,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> ChatSession:
        response = self.openai_client.with_raw_response.beta.chatkit.sessions.cancel(
            session_id=session_id,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        data = ChatSession(**json.loads(response.text))
        data._headers = response.headers

        return data


class ChatKitThreads(APIResource):
    def __init__(self, client: XimaAi) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    def retrieve(
        self,
        thread_id: str,
        *,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> ChatKitThread:
        response = self.openai_client.with_raw_response.beta.chatkit.threads.retrieve(
            thread_id=thread_id,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        data = ChatKitThread(**json.loads(response.text))
        data._headers = response.headers
        return data

    def list(
        self,
        *,
        after: Union[str, Omit] = omit,
        before: Union[str, Omit] = omit,
        limit: Union[int, Omit] = omit,
        order: Union[Omit, Literal["asc", "desc"]] = omit,
        user: Union[str, Omit] = omit,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> ChatKitThreadList:
        response = self.openai_client.with_raw_response.beta.chatkit.threads.list(
            after=after,
            before=before,
            limit=limit,
            order=order,
            user=user,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        data = ChatKitThreadList(**json.loads(response.text))
        data._headers = response.headers
        return data

    def delete(
        self,
        thread_id: str,
        *,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> ThreadDeleteResponse:
        response = self.openai_client.with_raw_response.beta.chatkit.threads.delete(
            thread_id=thread_id,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        data = ThreadDeleteResponse(**json.loads(response.text))
        data._headers = response.headers
        return data

    def list_items(
        self,
        thread_id: str,
        *,
        after: Union[str, Omit] = omit,
        before: Union[str, Omit] = omit,
        limit: Union[int, Omit] = omit,
        order: Union[Omit, Literal["asc", "desc"]] = omit,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> Any:
        response = self.openai_client.beta.chatkit.threads.list_items(
            thread_id=thread_id,
            after=after,
            before=before,
            limit=limit,
            order=order,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        return response


class AsyncChatKit(AsyncAPIResource):
    def __init__(self, client: AsyncPortkey) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client
        self.sessions = AsyncChatKitSessions(client)
        self.threads = AsyncChatKitThreads(client)

    async def upload_file(
        self,
        *,
        file: FileTypes,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> ChatKitUploadFileResponse:
        response = await self.openai_client.beta.chatkit.upload_file(
            file=file,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        return response


class AsyncChatKitSessions(AsyncAPIResource):
    def __init__(self, client: AsyncPortkey) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    async def create(
        self,
        *,
        user: str,
        workflow: ChatSessionWorkflowParam,
        chatkit_configuration: Union[ChatSessionChatKitConfigurationParam, Omit] = omit,
        expires_after: Union[ChatSessionExpiresAfterParam, Omit] = omit,
        rate_limits: Union[ChatSessionRateLimitsParam, Omit] = omit,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> ChatSession:
        response = (
            await self.openai_client.with_raw_response.beta.chatkit.sessions.create(
                user=user,
                workflow=workflow,
                chatkit_configuration=chatkit_configuration,
                expires_after=expires_after,
                rate_limits=rate_limits,
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            )
        )
        data = ChatSession(**json.loads(response.text))
        data._headers = response.headers

        return data

    async def cancel(
        self,
        session_id: str,
        *,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> ChatSession:
        response = (
            await self.openai_client.with_raw_response.beta.chatkit.sessions.cancel(
                session_id=session_id,
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            )
        )
        data = ChatSession(**json.loads(response.text))
        data._headers = response.headers

        return data


class AsyncChatKitThreads(AsyncAPIResource):
    def __init__(self, client: AsyncPortkey) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    async def retrieve(
        self,
        thread_id: str,
        *,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> ChatKitThread:
        response = (
            await self.openai_client.with_raw_response.beta.chatkit.threads.retrieve(
                thread_id=thread_id,
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            )
        )
        data = ChatKitThread(**json.loads(response.text))
        data._headers = response.headers
        return data

    async def list(
        self,
        *,
        after: Union[str, Omit] = omit,
        before: Union[str, Omit] = omit,
        limit: Union[int, Omit] = omit,
        order: Union[Omit, Literal["asc", "desc"]] = omit,
        user: Union[str, Omit] = omit,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> ChatKitThreadList:
        response = await self.openai_client.with_raw_response.beta.chatkit.threads.list(
            after=after,
            before=before,
            limit=limit,
            order=order,
            user=user,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        data = ChatKitThreadList(**json.loads(response.text))
        data._headers = response.headers
        return data

    async def delete(
        self,
        thread_id: str,
        *,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> ThreadDeleteResponse:
        response = (
            await self.openai_client.with_raw_response.beta.chatkit.threads.delete(
                thread_id=thread_id,
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
            )
        )
        data = ThreadDeleteResponse(**json.loads(response.text))
        data._headers = response.headers
        return data

    async def list_items(
        self,
        thread_id: str,
        *,
        after: Union[str, Omit] = omit,
        before: Union[str, Omit] = omit,
        limit: Union[int, Omit] = omit,
        order: Union[Omit, Literal["asc", "desc"]] = omit,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> Any:
        response = await self.openai_client.beta.chatkit.threads.list_items(
            thread_id=thread_id,
            after=after,
            before=before,
            limit=limit,
            order=order,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        return response



import builtins
import json
from typing import Any, Dict, List, Literal, Optional

import httpx
from pydantic import BaseModel, PrivateAttr

from xima_ai._vendor.openai.types.evals.runs.output_item_retrieve_response import (
    Sample,
)
from xima_ai.api_resources.types.utils import parse_headers


class OutputItemRetrieveResponse(BaseModel, extra="allow"):
    id: Optional[str] = None
    created_at: Optional[int] = None
    datasource_item: Optional[Dict[str, object]] = None
    datasource_item_id: Optional[int] = None
    eval_id: Optional[str] = None
    object: Optional[Literal["eval.run.output_item"]] = None
    results: Optional[List[Dict[str, builtins.object]]] = None
    run_id: Optional[str] = None
    sample: Optional[Sample] = None
    status: Optional[str] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class OutputItemListResponse(BaseModel, extra="allow"):
    id: Optional[str] = None
    created_at: Optional[int] = None
    datasource_item: Optional[Dict[str, object]] = None
    datasource_item_id: Optional[int] = None
    eval_id: Optional[str] = None
    object: Optional[Literal["eval.run.output_item"]] = None
    results: Optional[List[Dict[str, builtins.object]]] = None
    run_id: Optional[str] = None
    sample: Optional[Sample] = None
    status: Optional[str] = None


class OutputItemListResponseList(BaseModel, extra="allow"):
    object: Optional[str] = None
    data: Optional[List[OutputItemListResponse]] = None
    has_more: Optional[bool] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["RealtimeMcpProtocolErrorParam"]


class RealtimeMcpProtocolErrorParam(TypedDict, total=False):
    code: Required[int]

    message: Required[str]

    type: Required[Literal["protocol_error"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["OutputAudioBufferClearEvent"]


class OutputAudioBufferClearEvent(BaseModel):
    type: Literal["output_audio_buffer.clear"]
    """The event type, must be `output_audio_buffer.clear`."""

    event_id: Optional[str] = None
    """The unique ID of the client event used for error handling."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .._types import FileTypes
from .video_size import VideoSize
from .video_model import VideoModel
from .video_seconds import VideoSeconds

__all__ = ["VideoCreateParams"]


class VideoCreateParams(TypedDict, total=False):
    prompt: Required[str]
    """Text prompt that describes the video to generate."""

    input_reference: FileTypes
    """Optional image reference that guides generation."""

    model: VideoModel
    """The video generation model to use. Defaults to `sora-2`."""

    seconds: VideoSeconds
    """Clip duration in seconds. Defaults to 4 seconds."""

    size: VideoSize
    """Output resolution formatted as width x height. Defaults to 720x1280."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from ..._models import BaseModel
from ..shared.reasoning_effort import ReasoningEffort
from ..responses.response_input_text import ResponseInputText
from ..responses.response_input_audio import ResponseInputAudio

__all__ = [
    "ScoreModelGrader",
    "Input",
    "InputContent",
    "InputContentOutputText",
    "InputContentInputImage",
    "SamplingParams",
]


class InputContentOutputText(BaseModel):
    text: str
    """The text output from the model."""

    type: Literal["output_text"]
    """The type of the output text. Always `output_text`."""


class InputContentInputImage(BaseModel):
    image_url: str
    """The URL of the image input."""

    type: Literal["input_image"]
    """The type of the image input. Always `input_image`."""

    detail: Optional[str] = None
    """The detail level of the image to be sent to the model.

    One of `high`, `low`, or `auto`. Defaults to `auto`.
    """


InputContent: TypeAlias = Union[
    str, ResponseInputText, InputContentOutputText, InputContentInputImage, ResponseInputAudio, List[object]
]


class Input(BaseModel):
    content: InputContent
    """Inputs to the model - can contain template strings."""

    role: Literal["user", "assistant", "system", "developer"]
    """The role of the message input.

    One of `user`, `assistant`, `system`, or `developer`.
    """

    type: Optional[Literal["message"]] = None
    """The type of the message input. Always `message`."""


class SamplingParams(BaseModel):
    max_completions_tokens: Optional[int] = None
    """The maximum number of tokens the grader model may generate in its response."""

    reasoning_effort: Optional[ReasoningEffort] = None
    """
    Constrains effort on reasoning for
    [reasoning models](https://platform.openai.com/docs/guides/reasoning). Currently
    supported values are `minimal`, `low`, `medium`, and `high`. Reducing reasoning
    effort can result in faster responses and fewer tokens used on reasoning in a
    response.
    """

    seed: Optional[int] = None
    """A seed value to initialize the randomness, during sampling."""

    temperature: Optional[float] = None
    """A higher temperature increases randomness in the outputs."""

    top_p: Optional[float] = None
    """An alternative to temperature for nucleus sampling; 1.0 includes all tokens."""


class ScoreModelGrader(BaseModel):
    input: List[Input]
    """The input text. This may include template strings."""

    model: str
    """The model to use for the evaluation."""

    name: str
    """The name of the grader."""

    type: Literal["score_model"]
    """The object type, which is always `score_model`."""

    range: Optional[List[float]] = None
    """The range of the score. Defaults to `[0, 1]`."""

    sampling_params: Optional[SamplingParams] = None
    """The sampling parameters for the model."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import Annotated, TypeAlias

from ..._utils import PropertyInfo
from .file_part import FilePart
from .image_part import ImagePart

__all__ = ["ChatKitUploadFileResponse"]

ChatKitUploadFileResponse: TypeAlias = Annotated[Union[FilePart, ImagePart], PropertyInfo(discriminator="type")]



from typing import Any, Dict, Optional

from xima_ai.api_resources.apis.api_resource import APIResource, AsyncAPIResource
from xima_ai.api_resources.base_client import APIClient, AsyncAPIClient


class GetMethod(APIResource):
    def __init__(self, client: APIClient) -> None:
        super().__init__(client)

    def create(
        self,
        path: str,
        headers: Optional[Dict[str, str]] = {},
        cast_to: Optional[Any] = None,
    ):
        return self._get(
            path=path,
            body={},
            params={},
            headers=headers,
            cast_to=cast_to,
            stream=False,
            stream_cls=None,
        )


class AsyncGetMethod(AsyncAPIResource):
    def __init__(self, client: AsyncAPIClient) -> None:
        super().__init__(client)

    async def create(
        self,
        path: str,
        headers: Optional[Dict[str, str]] = {},
        cast_to: Optional[Any] = None,
    ):
        return await self._get(
            path=path,
            body={},
            params={},
            headers=headers,
            cast_to=cast_to,
            stream=False,
            stream_cls=None,
        )



import json
from typing import Any, Literal, Optional, Union

from xima_ai._vendor.openai._types import Omit, omit
from xima_ai.api_resources.apis.api_resource import APIResource, AsyncAPIResource
from xima_ai.api_resources.client import AsyncPortkey, XimaAi
from xima_ai.api_resources.types.evals_runs_output_items_type import (
    OutputItemListResponseList,
    OutputItemRetrieveResponse,
)
from xima_ai.api_resources.types.evals_runs_type import (
    RunCancelResponse,
    RunCreateResponse,
    RunDeleteResponse,
    RunListResponseList,
    RunRetrieveResponse,
)
from xima_ai.api_resources.types.evals_types import (
    EvalCreateResponse,
    EvalDeleteResponse,
    EvalListResponseList,
    EvalRetrieveResponse,
    EvalUpdateResponse,
)
from xima_ai.api_resources.types.shared_types import Metadata


class Evals(APIResource):
    def __init__(self, client: XimaAi) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client
        self.runs = EvalsRuns(client)

    def create(
        self,
        *,
        data_source_config: Any,
        testing_criteria: Any,
        metadata: Union[Optional[Metadata], Omit] = omit,
        name: Union[str, Omit] = omit,
        **kwargs,
    ) -> EvalCreateResponse:
        response = self.openai_client.evals.create(
            data_source_config=data_source_config,
            testing_criteria=testing_criteria,
            metadata=metadata,
            name=name,
            extra_body=kwargs,
        )
        data = response

        return data  # type: ignore[return-value]

    def retrieve(
        self,
        eval_id: str,
    ) -> EvalRetrieveResponse:
        response = self.openai_client.with_raw_response.evals.retrieve(
            eval_id=eval_id,
        )

        data = EvalRetrieveResponse(**json.loads(response.text))
        data._headers = response.headers

        return data

    def update(
        self,
        eval_id: str,
        *,
        metadata: Union[Optional[Metadata], Omit] = omit,
        name: Union[str, Omit] = omit,
        **kwargs,
    ) -> EvalUpdateResponse:
        response = self.openai_client.with_raw_response.evals.update(
            eval_id=eval_id,
            metadata=metadata,
            name=name,
            extra_body=kwargs,
        )

        data = EvalUpdateResponse(**json.loads(response.text))
        data._headers = response.headers

        return data

    def list(
        self,
        *,
        after: Union[str, Omit] = omit,
        limit: Union[int, Omit] = omit,
        order: Union[Literal["asc", "desc"], Omit] = omit,
        order_by: Union[Literal["created_at", "updated_at"], Omit] = omit,
        **kwargs,
    ) -> EvalListResponseList:
        response = self.openai_client.with_raw_response.evals.list(
            after=after,
            limit=limit,
            order=order,
            order_by=order_by,
        )

        data = EvalListResponseList(**json.loads(response.text))
        data._headers = response.headers

        return data

    def delete(
        self,
        eval_id: str,
        **kwargs,
    ) -> EvalDeleteResponse:
        response = self.openai_client.with_raw_response.evals.delete(
            eval_id=eval_id,
            extra_body=kwargs,
        )

        data = EvalDeleteResponse(**json.loads(response.text))
        data._headers = response.headers

        return data


class EvalsRuns(APIResource):
    def __init__(self, client: XimaAi) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client
        self.output_items = OutputItems(client)

    def create(
        self,
        eval_id: str,
        *,
        data_source: Any,
        metadata: Union[Optional[Metadata], Omit] = omit,
        name: Union[str, Omit] = omit,
        **kwargs,
    ) -> RunCreateResponse:
        response = self.openai_client.with_raw_response.evals.runs.create(
            eval_id=eval_id,
            data_source=data_source,
            metadata=metadata,
            name=name,
            extra_body=kwargs,
        )
        data = RunCreateResponse(**json.loads(response.text))
        data._headers = response.headers

        return data

    def retrieve(
        self,
        run_id: str,
        *,
        eval_id: str,
    ) -> RunRetrieveResponse:
        response = self.openai_client.with_raw_response.evals.runs.retrieve(
            run_id=run_id,
            eval_id=eval_id,
        )
        data = RunRetrieveResponse(**json.loads(response.text))
        data._headers = response.headers

        return data

    def list(
        self,
        eval_id: str,
        *,
        after: Union[str, Omit] = omit,
        limit: Union[int, Omit] = omit,
        order: Union[Literal["asc", "desc"], Omit] = omit,
        status: Union[
            Literal["queued", "in_progress", "completed", "canceled", "failed"],
            Omit,
        ] = omit,
    ) -> RunListResponseList:
        response = self.openai_client.with_raw_response.evals.runs.list(
            eval_id=eval_id,
            after=after,
            limit=limit,
            order=order,
            status=status,
        )
        data = RunListResponseList(**json.loads(response.text))
        data._headers = response.headers

        return data

    def delete(
        self,
        run_id: str,
        *,
        eval_id: str,
        **kwargs,
    ) -> RunDeleteResponse:
        response = self.openai_client.with_raw_response.evals.runs.delete(
            run_id=run_id,
            eval_id=eval_id,
            extra_body=kwargs,
        )
        data = RunDeleteResponse(**json.loads(response.text))
        data._headers = response.headers

        return data

    def cancel(
        self,
        run_id: str,
        *,
        eval_id: str,
        **kwargs,
    ) -> RunCancelResponse:
        response = self.openai_client.with_raw_response.evals.runs.cancel(
            run_id=run_id,
            eval_id=eval_id,
            extra_body=kwargs,
        )
        data = RunCancelResponse(**json.loads(response.text))
        data._headers = response.headers

        return data


class OutputItems(APIResource):
    def __init__(self, client: XimaAi) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    def retrieve(
        self,
        output_item_id: str,
        *,
        eval_id: str,
        run_id: str,
    ) -> OutputItemRetrieveResponse:
        response = (
            self.openai_client.with_raw_response.evals.runs.output_items.retrieve(
                output_item_id=output_item_id,
                eval_id=eval_id,
                run_id=run_id,
            )
        )
        data = OutputItemRetrieveResponse(**json.loads(response.text))
        data._headers = response.headers

        return data

    def list(
        self,
        run_id: str,
        *,
        eval_id: str,
        after: Union[str, Omit] = omit,
        limit: Union[int, Omit] = omit,
        order: Union[Literal["asc", "desc"], Omit] = omit,
        status: Union[Literal["fail", "pass"], Omit] = omit,
    ) -> OutputItemListResponseList:
        response = self.openai_client.with_raw_response.evals.runs.output_items.list(
            run_id=run_id,
            eval_id=eval_id,
            after=after,
            limit=limit,
            order=order,
            status=status,
        )
        data = OutputItemListResponseList(**json.loads(response.text))
        data._headers = response.headers

        return data


class AsyncEvals(AsyncAPIResource):
    def __init__(self, client: AsyncPortkey) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client
        self.runs = AsyncEvalsRuns(client)

    async def create(
        self,
        *,
        data_source_config: Any,
        testing_criteria: Any,
        metadata: Union[Optional[Metadata], Omit] = omit,
        name: Union[str, Omit] = omit,
        **kwargs,
    ) -> EvalCreateResponse:
        response = await self.openai_client.evals.create(
            data_source_config=data_source_config,
            testing_criteria=testing_criteria,
            metadata=metadata,
            name=name,
            extra_body=kwargs,
        )
        data = response

        return data  # type: ignore[return-value]

    async def retrieve(
        self,
        eval_id: str,
    ) -> EvalRetrieveResponse:
        response = await self.openai_client.with_raw_response.evals.retrieve(
            eval_id=eval_id,
        )

        data = EvalRetrieveResponse(**json.loads(response.text))
        data._headers = response.headers

        return data

    async def update(
        self,
        eval_id: str,
        *,
        metadata: Union[Optional[Metadata], Omit] = omit,
        name: Union[str, Omit] = omit,
        **kwargs,
    ) -> EvalUpdateResponse:
        response = await self.openai_client.with_raw_response.evals.update(
            eval_id=eval_id,
            metadata=metadata,
            name=name,
            extra_body=kwargs,
        )

        data = EvalUpdateResponse(**json.loads(response.text))
        data._headers = response.headers

        return data

    async def list(
        self,
        *,
        after: Union[str, Omit] = omit,
        limit: Union[int, Omit] = omit,
        order: Union[Literal["asc", "desc"], Omit] = omit,
        order_by: Union[Literal["created_at", "updated_at"], Omit] = omit,
        **kwargs,
    ) -> EvalListResponseList:
        response = await self.openai_client.with_raw_response.evals.list(
            after=after,
            limit=limit,
            order=order,
            order_by=order_by,
        )

        data = EvalListResponseList(**json.loads(response.text))
        data._headers = response.headers

        return data

    async def delete(
        self,
        eval_id: str,
        **kwargs,
    ) -> EvalDeleteResponse:
        response = await self.openai_client.with_raw_response.evals.delete(
            eval_id=eval_id,
            extra_body=kwargs,
        )

        data = EvalDeleteResponse(**json.loads(response.text))
        data._headers = response.headers

        return data


class AsyncEvalsRuns(AsyncAPIResource):
    def __init__(self, client: AsyncPortkey) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client
        self.output_items = AsyncOutputItems(client)

    async def create(
        self,
        eval_id: str,
        *,
        data_source: Any,
        metadata: Union[Optional[Metadata], Omit] = omit,
        name: Union[str, Omit] = omit,
        **kwargs,
    ) -> RunCreateResponse:
        response = await self.openai_client.with_raw_response.evals.runs.create(
            eval_id=eval_id,
            data_source=data_source,
            metadata=metadata,
            name=name,
            extra_body=kwargs,
        )
        data = RunCreateResponse(**json.loads(response.text))
        data._headers = response.headers

        return data

    async def retrieve(
        self,
        run_id: str,
        *,
        eval_id: str,
    ) -> RunRetrieveResponse:
        response = await self.openai_client.with_raw_response.evals.runs.retrieve(
            run_id=run_id,
            eval_id=eval_id,
        )
        data = RunRetrieveResponse(**json.loads(response.text))
        data._headers = response.headers

        return data

    async def list(
        self,
        eval_id: str,
        *,
        after: Union[str, Omit] = omit,
        limit: Union[int, Omit] = omit,
        order: Union[Literal["asc", "desc"], Omit] = omit,
        status: Union[
            Literal["queued", "in_progress", "completed", "canceled", "failed"],
            Omit,
        ] = omit,
    ) -> RunListResponseList:
        response = await self.openai_client.with_raw_response.evals.runs.list(
            eval_id=eval_id,
            after=after,
            limit=limit,
            order=order,
            status=status,
        )
        data = RunListResponseList(**json.loads(response.text))
        data._headers = response.headers

        return data

    async def delete(
        self,
        run_id: str,
        *,
        eval_id: str,
        **kwargs,
    ) -> RunDeleteResponse:
        response = await self.openai_client.with_raw_response.evals.runs.delete(
            run_id=run_id,
            eval_id=eval_id,
            extra_body=kwargs,
        )
        data = RunDeleteResponse(**json.loads(response.text))
        data._headers = response.headers

        return data

    async def cancel(
        self,
        run_id: str,
        *,
        eval_id: str,
        **kwargs,
    ) -> RunCancelResponse:
        response = await self.openai_client.with_raw_response.evals.runs.cancel(
            run_id=run_id,
            eval_id=eval_id,
            extra_body=kwargs,
        )
        data = RunCancelResponse(**json.loads(response.text))
        data._headers = response.headers

        return data


class AsyncOutputItems(AsyncAPIResource):
    def __init__(self, client: AsyncPortkey) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    async def retrieve(
        self,
        output_item_id: str,
        *,
        eval_id: str,
        run_id: str,
    ) -> OutputItemRetrieveResponse:
        response = (
            await self.openai_client.with_raw_response.evals.runs.output_items.retrieve(
                output_item_id=output_item_id,
                eval_id=eval_id,
                run_id=run_id,
            )
        )
        data = OutputItemRetrieveResponse(**json.loads(response.text))
        data._headers = response.headers

        return data

    async def list(
        self,
        run_id: str,
        *,
        eval_id: str,
        after: Union[str, Omit] = omit,
        limit: Union[int, Omit] = omit,
        order: Union[Literal["asc", "desc"], Omit] = omit,
        status: Union[Literal["fail", "pass"], Omit] = omit,
    ) -> OutputItemListResponseList:
        response = (
            await self.openai_client.with_raw_response.evals.runs.output_items.list(
                run_id=run_id,
                eval_id=eval_id,
                after=after,
                limit=limit,
                order=order,
                status=status,
            )
        )
        data = OutputItemListResponseList(**json.loads(response.text))
        data._headers = response.headers

        return data



"""Strands integration: thin adapter over Portkey Async client.

This module is only imported when users explicitly do:

    from portkey_ai.integrations.strands import PortkeyModel

It requires the optional dependency "strands-agents". Recommend install:

    pip install 'portkey-ai[strands]'

Design:
- Keep this adapter tiny. Heavy lifting (OpenAI-compatible streaming, etc.)
  is already provided by Portkey's SDK. We only:
  - Map Strands Messages/Tools -> OpenAI-compatible request.
  - Translate OpenAI-style stream chunks -> Strands StreamEvent.
  - Expose a class compatible with Strands `Model` interface.
"""

from __future__ import annotations

import json
import uuid
from typing import (
    Any,
    AsyncGenerator,
    AsyncIterator,
    Optional,
    TYPE_CHECKING,
    cast,
    List,
    Type,
    TypeVar,
    Union,
)

from portkey_ai import AsyncPortkey

if TYPE_CHECKING:  # Only used for static typing; no runtime import of Strands
    from strands.types.content import Messages
    from strands.types.tools import ToolSpec
    from strands.types.streaming import StreamEvent

T = TypeVar("T")

try:
    from strands.models.model import Model as _StrandsModel  # type: ignore

    _HAS_STRANDS = True
except Exception:
    _HAS_STRANDS = False

    class _StrandsModel:  # type: ignore
        pass


def _map_tools_to_openai(
    tool_specs: Optional[list[ToolSpec]],
) -> Optional[list[dict[str, Any]]]:
    if not tool_specs:
        return None
    tools: list[dict[str, Any]] = []
    for spec in tool_specs:
        json_schema = spec["inputSchema"]["json"]
        properties = json_schema.get("properties", {})
        # Drop default=None fields for OpenAI compatibility
        cleaned_properties = {
            key: {
                k: v
                for k, v in definition.items()
                if not (k == "default" and v is None)
            }
            for key, definition in properties.items()
        }
        tools.append(
            {
                "type": "function",
                "function": {
                    "name": spec["name"],
                    "description": spec.get("description", ""),
                    "parameters": {
                        "type": "object",
                        "properties": cleaned_properties,
                        "required": json_schema.get("required", []),
                    },
                },
            }
        )
    return tools


class _MessageFormatter:
    """Formats Strands messages into OpenAI-compatible messages.

    Tracks current tool use id to correctly emit tool result messages.
    """

    def __init__(self) -> None:
        self._current_tool_use_id: Optional[str] = None

    def format_messages(
        self, messages: "Messages", system_prompt: Optional[str]
    ) -> list[dict[str, Any]]:
        formatted: list[dict[str, Any]] = []

        if system_prompt:
            formatted.append({"role": "system", "content": system_prompt})

        for message in messages:
            role = message.get("role")
            content = message.get("content")
            if role not in ("user", "assistant"):
                continue

            if isinstance(content, str):
                formatted.append({"role": role, "content": content})
                continue

            if isinstance(content, list):
                for part in content:
                    if (
                        isinstance(part, dict)
                        and "text" in part
                        and isinstance(part["text"], str)
                    ):
                        formatted.append({"role": role, "content": part["text"]})
                    elif isinstance(part, dict) and "toolUse" in part:
                        formatted.append(self._format_tool_use_part(part))
                    elif (
                        isinstance(part, dict)
                        and "toolResult" in part
                        and self._current_tool_use_id is not None
                    ):
                        formatted.append(self._format_tool_result_part(part))

        return formatted

    def _format_tool_use_part(self, part: dict[str, Any]) -> dict[str, Any]:
        tool_use = part["toolUse"]
        tool_use_id = tool_use.get("toolUseId") or f"tooluse-{uuid.uuid4().hex[:8]}"
        self._current_tool_use_id = tool_use_id
        return {
            "role": "assistant",
            "tool_calls": [
                {
                    "id": tool_use_id,
                    "type": "function",
                    "function": {
                        "name": tool_use.get("name"),
                        "arguments": json.dumps(tool_use.get("input", {})),
                    },
                }
            ],
            "content": None,
        }

    def _format_tool_result_part(self, part: dict[str, Any]) -> dict[str, Any]:
        # Strands toolResult content is an array of blocks; concatenate text blocks
        content_blocks = part["toolResult"].get("content", [])
        text_parts = [
            blk["text"]
            for blk in content_blocks
            if isinstance(blk, dict) and "text" in blk
        ]
        result_text = " ".join(text_parts)
        return {
            "role": "tool",
            "tool_call_id": self._current_tool_use_id,
            "content": result_text,
        }


def _get_attr(obj: Any, name: str, default: Any = None) -> Any:
    try:
        return getattr(obj, name)
    except Exception:
        return cast(dict, obj).get(name, default)


def _format_chunk_to_stream_event(
    event: Any,
    state: dict[str, Optional[str]],
) -> "StreamEvent":
    """Translate an OpenAI-style ChatCompletionChunk into a Strands StreamEvent.

    Maintains tool use start/accumulation/stop using `state`.
    """
    choices = _get_attr(event, "choices") or [{}]
    choice0 = choices[0] if choices else {}
    delta = _get_attr(choice0, "delta") or {}

    # Tool calls
    tool_calls = _get_attr(delta, "tool_calls")
    if tool_calls:
        tool_call0 = tool_calls[0]
        function = _get_attr(tool_call0, "function") or {}
        tool_name = _get_attr(function, "name")
        arguments_chunk = _get_attr(function, "arguments", "") or ""
        call_type = _get_attr(tool_call0, "type")

        if tool_name and call_type and not state.get("tool_name"):
            state["tool_name"] = tool_name
            state["tool_use_id"] = f"{tool_name}-{uuid.uuid4().hex[:6]}"
            return cast(
                "StreamEvent",
                {
                    "contentBlockStart": {
                        "start": {
                            "toolUse": {
                                "name": tool_name,
                                "toolUseId": state["tool_use_id"],
                            }
                        }
                    }
                },
            )

        if arguments_chunk:
            return cast(
                "StreamEvent",
                {
                    "contentBlockDelta": {
                        "delta": {"toolUse": {"input": arguments_chunk}}
                    }
                },
            )

    finish_reason = str(_get_attr(choice0, "finish_reason", "") or "").lower()
    if finish_reason in ("tool_calls", "tool_use"):
        return cast(
            "StreamEvent",
            {
                "contentBlockStop": {
                    "name": state.get("tool_name"),
                    "toolUseId": state.get("tool_use_id"),
                }
            },
        )

    content_text = _get_attr(delta, "content")
    if content_text:
        return cast(
            "StreamEvent", {"contentBlockDelta": {"delta": {"text": content_text}}}
        )

    usage = _get_attr(event, "usage")
    if usage:
        return cast(
            "StreamEvent",
            {
                "metadata": {
                    "metrics": {"latencyMs": 0},
                    "usage": {
                        "inputTokens": _get_attr(usage, "prompt_tokens", 0) or 0,
                        "outputTokens": _get_attr(usage, "completion_tokens", 0) or 0,
                        "totalTokens": _get_attr(usage, "total_tokens", 0) or 0,
                    },
                }
            },
        )

    return cast("StreamEvent", {})


class PortkeyStrands(_StrandsModel):  # type: ignore[misc]
    """Strands `Model` adapter backed by Portkey Async client."""

    def __init__(self, **config: dict[str, Any]) -> None:  # type: ignore[override]
        if not _HAS_STRANDS:
            raise ImportError(
                "strands-agents is not installed. Install with: pip install 'portkey-ai[strands]'"
            )
        # Store config as-is for Strands get_config/update_config APIs
        self._config: dict[str, Any] = {"streaming": True}
        self.update_config(**config)

        self._provider_name: str = str(self._config.get("provider", "openai")).lower()
        self._client = AsyncPortkey(
            api_key=self._config.get("api_key"),
            virtual_key=self._config.get("virtual_key"),
            base_url=self._config.get("base_url"),
        )

    def update_config(self, **model_config: dict[str, Any]) -> None:  # type: ignore[override]
        self._config.update(model_config)

    def get_config(self) -> dict[str, Any]:  # type: ignore[override]
        return self._config

    def _allow_tool_use(self) -> bool:
        if self._provider_name == "openai":
            return True
        if self._provider_name == "bedrock":
            model_id = str(self._config.get("model_id", "")).lower()
            return "anthropic" in model_id
        return False

    async def stream(
        self,
        messages: "Messages",
        tool_specs: Optional[list["ToolSpec"]] = None,
        system_prompt: Optional[str] = None,
        **kwargs: Any,
    ) -> AsyncGenerator["StreamEvent", None]:  # type: ignore[override]
        formatter = _MessageFormatter()
        formatted_messages = formatter.format_messages(messages, system_prompt)

        request: dict[str, Any] = {
            "messages": formatted_messages,
            "model": self._config.get("model_id"),
            "stream": True,
        }

        if tool_specs and self._allow_tool_use():
            tools = _map_tools_to_openai(tool_specs)
            if tools:
                request["tools"] = tools
                request["tool_choice"] = "auto"

        state: dict[str, Optional[str]] = {"tool_use_id": None, "tool_name": None}

        yield cast("StreamEvent", {"messageStart": {"role": "assistant"}})
        # Await the creation to obtain an async iterator for streaming and cast for mypy
        stream_obj = await self._client.chat.completions.create(**request)  # type: ignore[arg-type]
        stream_iter = cast(AsyncIterator[Any], stream_obj)
        async for chunk in stream_iter:
            event = _format_chunk_to_stream_event(chunk, state)
            if event:
                yield event

            if (
                isinstance(event, dict)
                and "contentBlockStop" in event
                and state.get("tool_use_id")
            ):
                # Signal immediate message stop for tool use to let Strands hand over to tool executor
                yield cast("StreamEvent", {"messageStop": {"stopReason": "tool_use"}})

        # Reset internal state per call
        state["tool_use_id"] = None
        state["tool_name"] = None

    async def structured_output(
        self,
        output_model: "Type[T]",
        prompt: List[dict[str, Any]],
        system_prompt: Optional[str] = None,
        **kwargs: Any,
    ) -> AsyncGenerator[dict[str, Union["T", Any]], None]:  # type: ignore[override]
        """Placeholder to satisfy Strands Model abstract requirements.

        Note: PortkeyStrands currently focuses on streaming text/tool events via `stream`.
        Structured output is not yet implemented. This method exists to allow the
        class to be instantiated by Strands without raising an abstract class error.
        """
        raise NotImplementedError(
            "PortkeyStrands.structured_output is not implemented yet. "
            "Use model.stream(...) for streaming responses."
        )


__all__ = ["PortkeyStrands"]



from __future__ import annotations

from typing import Any, Dict, List, Mapping, Optional, Union

import httpx

from xima_ai.api_resources import apis
from xima_ai.api_resources.base_client import APIClient, AsyncAPIClient
from xima_ai.api_resources.global_constants import (
    OPEN_AI_API_KEY,
)

from .._vendor.openai import AsyncOpenAI, OpenAI


class XimaAi(APIClient):
    completions: apis.Completion
    chat: apis.ChatCompletion
    generations: apis.Generations
    prompts: apis.Prompts
    embeddings: apis.Embeddings
    feedback: apis.Feedback
    images: apis.Images
    files: apis.MainFiles
    models: apis.Models
    moderations: apis.Moderations
    audio: apis.Audio
    batches: apis.Batches
    fine_tuning: apis.FineTuning
    vector_stores: apis.VectorStores
    responses: apis.Responses
    webhooks: apis.Webhooks
    evals: apis.Evals
    containers: apis.Containers
    admin: apis.Admin
    uploads: apis.Uploads
    configs: apis.Configs
    api_keys: apis.ApiKeys
    virtual_keys: apis.VirtualKeys
    logs: apis.Logs
    labels: apis.Labels
    collections: apis.Collections
    integrations: apis.Integrations
    providers: apis.Providers
    guardrails: apis.Guardrails
    realtime: apis.MainRealtime
    conversations: apis.Conversations
    videos: apis.Videos

    class beta:
        assistants: apis.Assistants
        threads: apis.Threads
        chat: apis.BetaChat
        realtime: apis.BetaRealtime
        chatkit: apis.ChatKit

        def __init__(self, client: Portkey) -> None:
            self.assistants = apis.Assistants(client)
            self.threads = apis.Threads(client)
            self.chat = apis.BetaChat(client)
            self.realtime = apis.BetaRealtime(client)
            self.chatkit = apis.ChatKit(client)

    def __init__(
        self,
        *,
        api_key: Optional[str] = None,
        base_url: Optional[str] = None,
        virtual_key: Optional[str] = None,
        websocket_base_url: Optional[Union[str, httpx.URL]] = None,
        webhook_secret: Optional[str] = None,
        config: Optional[Union[Mapping, str]] = None,
        provider: Optional[str] = None,
        trace_id: Optional[str] = None,
        metadata: Union[Optional[dict[str, str]], str] = None,
        cache_namespace: Optional[str] = None,
        debug: Optional[bool] = None,
        cache_force_refresh: Optional[bool] = None,
        custom_host: Optional[str] = None,
        forward_headers: Optional[List[str]] = None,
        instrumentation: Optional[bool] = None,
        openai_project: Optional[str] = None,
        openai_organization: Optional[str] = None,
        aws_secret_access_key: Optional[str] = None,
        aws_access_key_id: Optional[str] = None,
        aws_session_token: Optional[str] = None,
        aws_region: Optional[str] = None,
        vertex_project_id: Optional[str] = None,
        vertex_region: Optional[str] = None,
        workers_ai_account_id: Optional[str] = None,
        azure_resource_name: Optional[str] = None,
        azure_deployment_id: Optional[str] = None,
        azure_api_version: Optional[str] = None,
        azure_endpoint_name: Optional[str] = None,
        huggingface_base_url: Optional[str] = None,
        http_client: Optional[httpx.Client] = None,
        request_timeout: Optional[int] = None,
        strict_open_ai_compliance: Optional[bool] = False,
        anthropic_beta: Optional[str] = None,
        anthropic_version: Optional[str] = None,
        mistral_fim_completion: Optional[str] = None,
        vertex_storage_bucket_name: Optional[str] = None,
        provider_file_name: Optional[str] = None,
        provider_model: Optional[str] = None,
        aws_s3_bucket: Optional[str] = None,
        aws_s3_object_key: Optional[str] = None,
        aws_bedrock_model: Optional[str] = None,
        fireworks_account_id: Optional[str] = None,
        calculate_audio_duration: Optional[bool] = True,
        **kwargs,
    ) -> None:
        super().__init__(
            api_key=api_key,
            base_url=base_url,
            virtual_key=virtual_key,
            websocket_base_url=websocket_base_url,
            webhook_secret=webhook_secret,
            config=config,
            provider=provider,
            trace_id=trace_id,
            metadata=metadata,
            debug=debug,
            cache_force_refresh=cache_force_refresh,
            custom_host=custom_host,
            forward_headers=forward_headers,
            instrumentation=instrumentation,
            openai_project=openai_project,
            openai_organization=openai_organization,
            aws_secret_access_key=aws_secret_access_key,
            aws_access_key_id=aws_access_key_id,
            aws_session_token=aws_session_token,
            aws_region=aws_region,
            vertex_project_id=vertex_project_id,
            vertex_region=vertex_region,
            workers_ai_account_id=workers_ai_account_id,
            azure_resource_name=azure_resource_name,
            azure_deployment_id=azure_deployment_id,
            azure_api_version=azure_api_version,
            azure_endpoint_name=azure_endpoint_name,
            huggingface_base_url=huggingface_base_url,
            cache_namespace=cache_namespace,
            http_client=http_client,
            request_timeout=request_timeout,
            strict_open_ai_compliance=strict_open_ai_compliance,
            anthropic_beta=anthropic_beta,
            anthropic_version=anthropic_version,
            mistral_fim_completion=mistral_fim_completion,
            vertex_storage_bucket_name=vertex_storage_bucket_name,
            provider_file_name=provider_file_name,
            provider_model=provider_model,
            aws_s3_bucket=aws_s3_bucket,
            aws_s3_object_key=aws_s3_object_key,
            aws_bedrock_model=aws_bedrock_model,
            fireworks_account_id=fireworks_account_id,
            calculate_audio_duration=calculate_audio_duration,
            **kwargs,
        )

        self.openai_client = OpenAI(
            api_key=OPEN_AI_API_KEY,
            base_url=self.base_url,
            default_headers=self.allHeaders,
            http_client=http_client,
            max_retries=1,
            websocket_base_url=self.websocket_base_url,
            webhook_secret=self.webhook_secret,
        )

        self.completions = apis.Completion(self)
        self.chat = apis.ChatCompletion(self)
        self.generations = apis.Generations(self)
        self.prompts = apis.Prompts(self)
        self.embeddings = apis.Embeddings(self)
        self.feedback = apis.Feedback(self)
        self.images = apis.Images(self)
        self.files = apis.MainFiles(self)
        self.models = apis.Models(self)
        self.moderations = apis.Moderations(self)
        self.audio = apis.Audio(self)
        self.batches = apis.Batches(self)
        self.fine_tuning = apis.FineTuning(self)
        self.vector_stores = apis.VectorStores(self)
        self.responses = apis.Responses(self)
        self.webhooks = apis.Webhooks(self)
        self.evals = apis.Evals(self)
        self.containers = apis.Containers(self)
        self.admin = apis.Admin(self)
        self.uploads = apis.Uploads(self)
        self.configs = apis.Configs(self)
        self.api_keys = apis.ApiKeys(self)
        self.virtual_keys = apis.VirtualKeys(self)
        self.logs = apis.Logs(self)
        self.labels = apis.Labels(self)
        self.collections = apis.Collections(self)
        self.integrations = apis.Integrations(self)
        self.providers = apis.Providers(self)
        self.guardrails = apis.Guardrails(self)
        self.realtime = apis.MainRealtime(self)
        self.conversations = apis.Conversations(self)
        self.videos = apis.Videos(self)
        self.beta = self.beta(self)  # type: ignore

        if self.instrumentation:
            try:
                from xima_ai.api_resources.instrumentation import (
                    initialize_instrumentation,
                )
            except ImportError:
                raise ImportError(
                    """Please install opentelemetry for instrumentation,
                    you can use `pip install 'portkey-ai[instrumentation]'`
                    to install"""
                )
            initialize_instrumentation(api_key=self.api_key, base_url=self.base_url)

    def copy(
        self,
        *,
        api_key: Optional[str] = None,
        base_url: Optional[str] = None,
        virtual_key: Optional[str] = None,
        websocket_base_url: Optional[Union[str, httpx.URL]] = None,
        webhook_secret: Optional[str] = None,
        config: Optional[Union[Mapping, str]] = None,
        provider: Optional[str] = None,
        trace_id: Optional[str] = None,
        metadata: Union[Optional[dict[str, str]], str] = None,
        cache_namespace: Optional[str] = None,
        debug: Optional[bool] = None,
        cache_force_refresh: Optional[bool] = None,
        custom_host: Optional[str] = None,
        forward_headers: Optional[List[str]] = None,
        instrumentation: Optional[bool] = None,
        openai_project: Optional[str] = None,
        openai_organization: Optional[str] = None,
        aws_secret_access_key: Optional[str] = None,
        aws_access_key_id: Optional[str] = None,
        aws_session_token: Optional[str] = None,
        aws_region: Optional[str] = None,
        vertex_project_id: Optional[str] = None,
        vertex_region: Optional[str] = None,
        workers_ai_account_id: Optional[str] = None,
        azure_resource_name: Optional[str] = None,
        azure_deployment_id: Optional[str] = None,
        azure_api_version: Optional[str] = None,
        azure_endpoint_name: Optional[str] = None,
        huggingface_base_url: Optional[str] = None,
        http_client: Optional[httpx.Client] = None,
        request_timeout: Optional[int] = None,
        strict_open_ai_compliance: Optional[bool] = False,
        anthropic_beta: Optional[str] = None,
        anthropic_version: Optional[str] = None,
        mistral_fim_completion: Optional[str] = None,
        vertex_storage_bucket_name: Optional[str] = None,
        provider_file_name: Optional[str] = None,
        provider_model: Optional[str] = None,
        aws_s3_bucket: Optional[str] = None,
        aws_s3_object_key: Optional[str] = None,
        aws_bedrock_model: Optional[str] = None,
        fireworks_account_id: Optional[str] = None,
        calculate_audio_duration: Optional[bool] = True,
        **kwargs,
    ) -> XimaAi:
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            virtual_key=virtual_key or self.virtual_key,
            websocket_base_url=websocket_base_url or self.websocket_base_url,
            webhook_secret=webhook_secret or self.webhook_secret,
            config=config or self.config,
            provider=provider or self.provider,
            trace_id=trace_id or self.trace_id,
            metadata=metadata or self.metadata,
            debug=debug or self.debug,
            cache_force_refresh=cache_force_refresh or self.cache_force_refresh,
            custom_host=custom_host or self.custom_host,
            forward_headers=forward_headers or self.forward_headers,
            instrumentation=instrumentation or self.instrumentation,
            openai_project=openai_project or self.openai_project,
            openai_organization=openai_organization or self.openai_organization,
            aws_secret_access_key=aws_secret_access_key or self.aws_secret_access_key,
            aws_access_key_id=aws_access_key_id or self.aws_access_key_id,
            aws_session_token=aws_session_token or self.aws_session_token,
            aws_region=aws_region or self.aws_region,
            vertex_project_id=vertex_project_id or self.vertex_project_id,
            vertex_region=vertex_region or self.vertex_region,
            workers_ai_account_id=workers_ai_account_id or self.workers_ai_account_id,
            azure_resource_name=azure_resource_name or self.azure_resource_name,
            azure_deployment_id=azure_deployment_id or self.azure_deployment_id,
            azure_api_version=azure_api_version or self.azure_api_version,
            azure_endpoint_name=azure_endpoint_name or self.azure_endpoint_name,
            huggingface_base_url=huggingface_base_url or self.huggingface_base_url,
            cache_namespace=cache_namespace or self.cache_namespace,
            http_client=http_client or self._client,
            request_timeout=request_timeout or self.request_timeout,
            strict_open_ai_compliance=strict_open_ai_compliance
            or self.strict_open_ai_compliance,
            anthropic_beta=anthropic_beta or self.anthropic_beta,
            anthropic_version=anthropic_version or self.anthropic_version,
            mistral_fim_completion=mistral_fim_completion
            or self.mistral_fim_completion,
            vertex_storage_bucket_name=vertex_storage_bucket_name
            or self.vertex_storage_bucket_name,
            provider_file_name=provider_file_name or self.provider_file_name,
            provider_model=provider_model or self.provider_model,
            aws_s3_bucket=aws_s3_bucket or self.aws_s3_bucket,
            aws_s3_object_key=aws_s3_object_key or self.aws_s3_object_key,
            aws_bedrock_model=aws_bedrock_model or self.aws_bedrock_model,
            fireworks_account_id=fireworks_account_id or self.fireworks_account_id,
            calculate_audio_duration=calculate_audio_duration
            or self.calculate_audio_duration,
            **self.kwargs,
            **kwargs,
        )

    def post(self, url: str, **kwargs):
        return apis.Post(self).create(url=url, **kwargs)

    def get(
        self,
        *,
        path: str,
        headers: Optional[Dict[str, str]] = {},
        cast_to: Optional[Any] = None,
    ):
        return apis.GetMethod(self).create(path=path, headers=headers, cast_to=cast_to)

    def delete(
        self,
        *,
        path: str,
        headers: Optional[Dict[str, str]] = {},
        cast_to: Optional[Any] = None,
    ):
        return apis.DeleteMethod(self).create(
            path=path, headers=headers, cast_to=cast_to
        )

    def put(
        self,
        *,
        path: str,
        body: Optional[Dict[str, Any]] = {},
        headers: Optional[Dict[str, str]] = {},
        cast_to: Optional[Any] = None,
    ):
        return apis.PutMethod(self).create(
            path=path, body=body, headers=headers, cast_to=cast_to
        )

    with_options = copy


class AsyncPortkey(AsyncAPIClient):
    completions: apis.AsyncCompletion
    chat: apis.AsyncChatCompletion
    generations: apis.AsyncGenerations
    prompts: apis.AsyncPrompts
    embeddings: apis.AsyncEmbeddings
    feedback: apis.AsyncFeedback
    images: apis.AsyncImages
    files: apis.AsyncMainFiles
    models: apis.AsyncModels
    moderations: apis.AsyncModerations
    audio: apis.AsyncAudio
    batches: apis.AsyncBatches
    fine_tuning: apis.AsyncFineTuning
    vector_stores: apis.AsyncVectorStores
    responses: apis.AsyncResponses
    webhooks: apis.AsyncWebhooks
    evals: apis.AsyncEvals
    containers: apis.AsyncContainers
    admin: apis.AsyncAdmin
    uploads: apis.AsyncUploads
    configs: apis.AsyncConfigs
    api_keys: apis.AsyncApiKeys
    virtual_keys: apis.AsyncVirtualKeys
    logs: apis.AsyncLogs
    labels: apis.AsyncLabels
    collections: apis.AsyncCollections
    integrations: apis.AsyncIntegrations
    providers: apis.AsyncProviders
    guardrails: apis.AsyncGuardrails
    realtime: apis.AsyncMainRealtime
    conversations: apis.AsyncConversations
    videos: apis.AsyncVideos

    class beta:
        assistants: apis.AsyncAssistants
        threads: apis.AsyncThreads
        chat: apis.AsyncBetaChat
        realtime: apis.AsyncBetaRealtime
        chatkit: apis.AsyncChatKit

        def __init__(self, client: AsyncPortkey) -> None:
            self.assistants = apis.AsyncAssistants(client)
            self.threads = apis.AsyncThreads(client)
            self.chat = apis.AsyncBetaChat(client)
            self.realtime = apis.AsyncBetaRealtime(client)
            self.chatkit = apis.AsyncChatKit(client)

    def __init__(
        self,
        *,
        api_key: Optional[str] = None,
        base_url: Optional[str] = None,
        virtual_key: Optional[str] = None,
        websocket_base_url: Optional[Union[str, httpx.URL]] = None,
        webhook_secret: Optional[str] = None,
        config: Optional[Union[Mapping, str]] = None,
        provider: Optional[str] = None,
        trace_id: Optional[str] = None,
        metadata: Union[Optional[dict[str, str]], str] = None,
        cache_namespace: Optional[str] = None,
        debug: Optional[bool] = None,
        cache_force_refresh: Optional[bool] = None,
        custom_host: Optional[str] = None,
        forward_headers: Optional[List[str]] = None,
        instrumentation: Optional[bool] = None,
        openai_project: Optional[str] = None,
        openai_organization: Optional[str] = None,
        aws_secret_access_key: Optional[str] = None,
        aws_access_key_id: Optional[str] = None,
        aws_session_token: Optional[str] = None,
        aws_region: Optional[str] = None,
        vertex_project_id: Optional[str] = None,
        vertex_region: Optional[str] = None,
        workers_ai_account_id: Optional[str] = None,
        azure_resource_name: Optional[str] = None,
        azure_deployment_id: Optional[str] = None,
        azure_api_version: Optional[str] = None,
        azure_endpoint_name: Optional[str] = None,
        huggingface_base_url: Optional[str] = None,
        http_client: Optional[httpx.AsyncClient] = None,
        request_timeout: Optional[int] = None,
        strict_open_ai_compliance: Optional[bool] = False,
        anthropic_beta: Optional[str] = None,
        anthropic_version: Optional[str] = None,
        mistral_fim_completion: Optional[str] = None,
        vertex_storage_bucket_name: Optional[str] = None,
        provider_file_name: Optional[str] = None,
        provider_model: Optional[str] = None,
        aws_s3_bucket: Optional[str] = None,
        aws_s3_object_key: Optional[str] = None,
        aws_bedrock_model: Optional[str] = None,
        fireworks_account_id: Optional[str] = None,
        calculate_audio_duration: Optional[bool] = True,
        **kwargs,
    ) -> None:
        super().__init__(
            api_key=api_key,
            base_url=base_url,
            virtual_key=virtual_key,
            websocket_base_url=websocket_base_url,
            webhook_secret=webhook_secret,
            config=config,
            provider=provider,
            trace_id=trace_id,
            metadata=metadata,
            debug=debug,
            cache_force_refresh=cache_force_refresh,
            custom_host=custom_host,
            forward_headers=forward_headers,
            instrumentation=instrumentation,
            openai_project=openai_project,
            openai_organization=openai_organization,
            aws_secret_access_key=aws_secret_access_key,
            aws_access_key_id=aws_access_key_id,
            aws_session_token=aws_session_token,
            aws_region=aws_region,
            vertex_project_id=vertex_project_id,
            vertex_region=vertex_region,
            workers_ai_account_id=workers_ai_account_id,
            azure_resource_name=azure_resource_name,
            azure_deployment_id=azure_deployment_id,
            azure_api_version=azure_api_version,
            azure_endpoint_name=azure_endpoint_name,
            huggingface_base_url=huggingface_base_url,
            cache_namespace=cache_namespace,
            http_client=http_client,
            request_timeout=request_timeout,
            strict_open_ai_compliance=strict_open_ai_compliance,
            anthropic_beta=anthropic_beta,
            anthropic_version=anthropic_version,
            mistral_fim_completion=mistral_fim_completion,
            vertex_storage_bucket_name=vertex_storage_bucket_name,
            provider_file_name=provider_file_name,
            provider_model=provider_model,
            aws_s3_bucket=aws_s3_bucket,
            aws_s3_object_key=aws_s3_object_key,
            aws_bedrock_model=aws_bedrock_model,
            fireworks_account_id=fireworks_account_id,
            calculate_audio_duration=calculate_audio_duration,
            **kwargs,
        )

        self.openai_client = AsyncOpenAI(
            api_key=OPEN_AI_API_KEY,
            base_url=self.base_url,
            default_headers=self.allHeaders,
            http_client=http_client,
            max_retries=1,
            websocket_base_url=self.websocket_base_url,
            webhook_secret=self.webhook_secret,
        )

        self.completions = apis.AsyncCompletion(self)
        self.chat = apis.AsyncChatCompletion(self)
        self.generations = apis.AsyncGenerations(self)
        self.prompts = apis.AsyncPrompts(self)
        self.embeddings = apis.AsyncEmbeddings(self)
        self.feedback = apis.AsyncFeedback(self)
        self.images = apis.AsyncImages(self)
        self.files = apis.AsyncMainFiles(self)
        self.models = apis.AsyncModels(self)
        self.moderations = apis.AsyncModerations(self)
        self.audio = apis.AsyncAudio(self)
        self.batches = apis.AsyncBatches(self)
        self.fine_tuning = apis.AsyncFineTuning(self)
        self.vector_stores = apis.AsyncVectorStores(self)
        self.responses = apis.AsyncResponses(self)
        self.webhooks = apis.AsyncWebhooks(self)
        self.evals = apis.AsyncEvals(self)
        self.containers = apis.AsyncContainers(self)
        self.admin = apis.AsyncAdmin(self)
        self.uploads = apis.AsyncUploads(self)
        self.configs = apis.AsyncConfigs(self)
        self.api_keys = apis.AsyncApiKeys(self)
        self.virtual_keys = apis.AsyncVirtualKeys(self)
        self.logs = apis.AsyncLogs(self)
        self.labels = apis.AsyncLabels(self)
        self.collections = apis.AsyncCollections(self)
        self.integrations = apis.AsyncIntegrations(self)
        self.providers = apis.AsyncProviders(self)
        self.guardrails = apis.AsyncGuardrails(self)
        self.realtime = apis.AsyncMainRealtime(self)
        self.conversations = apis.AsyncConversations(self)
        self.videos = apis.AsyncVideos(self)
        self.beta = self.beta(self)  # type: ignore

        if self.instrumentation:
            try:
                from xima_ai.api_resources.instrumentation import (
                    initialize_instrumentation,
                )
            except ImportError:
                raise ImportError(
                    """Please install opentelemetry for instrumentation,
                    you can use `pip install 'portkey-ai[instrumentation]'`
                    to install"""
                )
            initialize_instrumentation(api_key=self.api_key, base_url=self.base_url)

    def copy(
        self,
        *,
        api_key: Optional[str] = None,
        base_url: Optional[str] = None,
        virtual_key: Optional[str] = None,
        websocket_base_url: Optional[Union[str, httpx.URL]] = None,
        webhook_secret: Optional[str] = None,
        config: Optional[Union[Mapping, str]] = None,
        provider: Optional[str] = None,
        trace_id: Optional[str] = None,
        metadata: Union[Optional[dict[str, str]], str] = None,
        cache_namespace: Optional[str] = None,
        debug: Optional[bool] = None,
        cache_force_refresh: Optional[bool] = None,
        custom_host: Optional[str] = None,
        forward_headers: Optional[List[str]] = None,
        instrumentation: Optional[bool] = None,
        openai_project: Optional[str] = None,
        openai_organization: Optional[str] = None,
        aws_secret_access_key: Optional[str] = None,
        aws_access_key_id: Optional[str] = None,
        aws_session_token: Optional[str] = None,
        aws_region: Optional[str] = None,
        vertex_project_id: Optional[str] = None,
        vertex_region: Optional[str] = None,
        workers_ai_account_id: Optional[str] = None,
        azure_resource_name: Optional[str] = None,
        azure_deployment_id: Optional[str] = None,
        azure_api_version: Optional[str] = None,
        azure_endpoint_name: Optional[str] = None,
        huggingface_base_url: Optional[str] = None,
        http_client: Optional[httpx.AsyncClient] = None,
        request_timeout: Optional[int] = None,
        strict_open_ai_compliance: Optional[bool] = False,
        anthropic_beta: Optional[str] = None,
        anthropic_version: Optional[str] = None,
        mistral_fim_completion: Optional[str] = None,
        vertex_storage_bucket_name: Optional[str] = None,
        provider_file_name: Optional[str] = None,
        provider_model: Optional[str] = None,
        aws_s3_bucket: Optional[str] = None,
        aws_s3_object_key: Optional[str] = None,
        aws_bedrock_model: Optional[str] = None,
        fireworks_account_id: Optional[str] = None,
        calculate_audio_duration: Optional[bool] = True,
        **kwargs,
    ) -> AsyncPortkey:
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            virtual_key=virtual_key or self.virtual_key,
            websocket_base_url=websocket_base_url or self.websocket_base_url,
            webhook_secret=webhook_secret or self.webhook_secret,
            config=config or self.config,
            provider=provider or self.provider,
            trace_id=trace_id or self.trace_id,
            metadata=metadata or self.metadata,
            debug=debug or self.debug,
            cache_force_refresh=cache_force_refresh or self.cache_force_refresh,
            custom_host=custom_host or self.custom_host,
            forward_headers=forward_headers or self.forward_headers,
            instrumentation=instrumentation or self.instrumentation,
            openai_project=openai_project or self.openai_project,
            openai_organization=openai_organization or self.openai_organization,
            aws_secret_access_key=aws_secret_access_key or self.aws_secret_access_key,
            aws_access_key_id=aws_access_key_id or self.aws_access_key_id,
            aws_session_token=aws_session_token or self.aws_session_token,
            aws_region=aws_region or self.aws_region,
            vertex_project_id=vertex_project_id or self.vertex_project_id,
            vertex_region=vertex_region or self.vertex_region,
            workers_ai_account_id=workers_ai_account_id or self.workers_ai_account_id,
            azure_resource_name=azure_resource_name or self.azure_resource_name,
            azure_deployment_id=azure_deployment_id or self.azure_deployment_id,
            azure_api_version=azure_api_version or self.azure_api_version,
            azure_endpoint_name=azure_endpoint_name or self.azure_endpoint_name,
            huggingface_base_url=huggingface_base_url or self.huggingface_base_url,
            cache_namespace=cache_namespace or self.cache_namespace,
            http_client=http_client or self._client,
            request_timeout=request_timeout or self.request_timeout,
            strict_open_ai_compliance=strict_open_ai_compliance
            or self.strict_open_ai_compliance,
            anthropic_beta=anthropic_beta or self.anthropic_beta,
            anthropic_version=anthropic_version or self.anthropic_version,
            mistral_fim_completion=mistral_fim_completion
            or self.mistral_fim_completion,
            vertex_storage_bucket_name=vertex_storage_bucket_name
            or self.vertex_storage_bucket_name,
            provider_file_name=provider_file_name or self.provider_file_name,
            provider_model=provider_model or self.provider_model,
            aws_s3_bucket=aws_s3_bucket or self.aws_s3_bucket,
            aws_s3_object_key=aws_s3_object_key or self.aws_s3_object_key,
            aws_bedrock_model=aws_bedrock_model or self.aws_bedrock_model,
            fireworks_account_id=fireworks_account_id or self.fireworks_account_id,
            calculate_audio_duration=calculate_audio_duration
            or self.calculate_audio_duration,
            **self.kwargs,
            **kwargs,
        )

    async def post(self, url: str, **kwargs):
        return await apis.AsyncPost(self).create(url=url, **kwargs)

    async def get(
        self,
        *,
        path: str,
        headers: Optional[Dict[str, str]] = {},
        cast_to: Optional[Any] = None,
    ):
        return await apis.AsyncGetMethod(self).create(
            path=path, headers=headers, cast_to=cast_to
        )

    async def delete(
        self,
        *,
        path: str,
        headers: Optional[Dict[str, str]] = {},
        cast_to: Optional[Any] = None,
    ):
        return await apis.AsyncDeleteMethod(self).create(
            path=path, headers=headers, cast_to=cast_to
        )

    async def put(
        self,
        *,
        path: str,
        body: Optional[Dict[str, Any]] = {},
        headers: Optional[Dict[str, str]] = {},
        cast_to: Optional[Any] = None,
    ):
        return await apis.AsyncPutMethod(self).create(
            path=path, body=body, headers=headers, cast_to=cast_to
        )

    with_options = copy



import json
from typing import Any, Literal, Optional, Union

import httpx

from xima_ai._vendor.openai.types.video_model import VideoModel
from xima_ai._vendor.openai.types.video_seconds import VideoSeconds
from xima_ai._vendor.openai.types.video_size import VideoSize
from xima_ai.api_resources.apis.api_resource import APIResource, AsyncAPIResource
from xima_ai.api_resources.client import AsyncPortkey, XimaAi
from xima_ai.api_resources.types.shared_types import Body, Headers, Query
from xima_ai.api_resources.types.videos_type import (
    Video,
    VideoDeleteResponse,
    VideoList,
)

from ..._vendor.openai._types import FileTypes, NotGiven, Omit, not_given, omit


class Videos(APIResource):
    def __init__(self, client: XimaAi) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    def create(
        self,
        *,
        prompt: str,
        input_reference: Union[FileTypes, Omit] = omit,
        model: Union[VideoModel, Omit] = omit,
        seconds: Union[VideoSeconds, Omit] = omit,
        size: Union[VideoSize, Omit] = omit,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> Video:
        response = self.openai_client.with_raw_response.videos.create(
            prompt=prompt,
            input_reference=input_reference,
            model=model,
            seconds=seconds,
            size=size,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        data = Video(**json.loads(response.text))
        data._headers = response.headers

        return data

    def create_and_poll(
        self,
        *,
        prompt: str,
        input_reference: Union[FileTypes, Omit] = omit,
        model: Union[VideoModel, Omit] = omit,
        seconds: Union[VideoSeconds, Omit] = omit,
        size: Union[VideoSize, Omit] = omit,
        poll_interval_ms: Union[int, Omit] = omit,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> Video:
        response = self.openai_client.videos.create_and_poll(
            prompt=prompt,
            input_reference=input_reference,
            model=model,
            seconds=seconds,
            size=size,
            poll_interval_ms=poll_interval_ms,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        return response  # type: ignore[return-value]

    def poll(
        self,
        video_id: str,
        *,
        poll_interval_ms: Union[int, Omit] = omit,
    ) -> Video:
        response = self.openai_client.videos.poll(
            video_id=video_id,
            poll_interval_ms=poll_interval_ms,
        )
        return response  # type: ignore[return-value]

    def retrieve(
        self,
        video_id: str,
        *,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> Video:
        response = self.openai_client.with_raw_response.videos.retrieve(
            video_id=video_id,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        data = Video(**json.loads(response.text))
        data._headers = response.headers

        return data

    def list(
        self,
        *,
        after: Union[str, Omit] = omit,
        limit: Union[int, Omit] = omit,
        order: Union[Literal["asc", "desc"], Omit] = omit,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> VideoList:
        response = self.openai_client.with_raw_response.videos.list(
            after=after,
            limit=limit,
            order=order,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        data = VideoList(**json.loads(response.text))
        data._headers = response.headers
        return data

    def delete(
        self,
        video_id: str,
        *,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> VideoDeleteResponse:
        response = self.openai_client.with_raw_response.videos.delete(
            video_id=video_id,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        data = VideoDeleteResponse(**json.loads(response.text))
        data._headers = response.headers
        return data

    def download_content(
        self,
        video_id: str,
        *,
        variant: Union[Literal["video", "thumbnail", "spritesheet"], Omit] = omit,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> Any:
        response = self.openai_client.videos.download_content(
            video_id=video_id,
            variant=variant,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        return response

    def remix(
        self,
        video_id: str,
        *,
        prompt: str,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> Video:
        response = self.openai_client.with_raw_response.videos.remix(
            video_id=video_id,
            prompt=prompt,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        data = Video(**json.loads(response.text))
        data._headers = response.headers
        return data


class AsyncVideos(AsyncAPIResource):
    def __init__(self, client: AsyncPortkey) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    async def create(
        self,
        *,
        prompt: str,
        input_reference: Union[FileTypes, Omit] = omit,
        model: Union[VideoModel, Omit] = omit,
        seconds: Union[VideoSeconds, Omit] = omit,
        size: Union[VideoSize, Omit] = omit,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> Video:
        response = await self.openai_client.with_raw_response.videos.create(
            prompt=prompt,
            input_reference=input_reference,
            model=model,
            seconds=seconds,
            size=size,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        data = Video(**json.loads(response.text))
        data._headers = response.headers

        return data

    async def create_and_poll(
        self,
        *,
        prompt: str,
        input_reference: Union[FileTypes, Omit] = omit,
        model: Union[VideoModel, Omit] = omit,
        seconds: Union[VideoSeconds, Omit] = omit,
        size: Union[VideoSize, Omit] = omit,
        poll_interval_ms: Union[int, Omit] = omit,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> Video:
        response = await self.openai_client.videos.create_and_poll(
            prompt=prompt,
            input_reference=input_reference,
            model=model,
            seconds=seconds,
            size=size,
            poll_interval_ms=poll_interval_ms,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        return response  # type: ignore[return-value]

    async def poll(
        self,
        video_id: str,
        *,
        poll_interval_ms: Union[int, Omit] = omit,
    ) -> Video:
        response = await self.openai_client.videos.poll(
            video_id=video_id,
            poll_interval_ms=poll_interval_ms,
        )
        return response  # type: ignore[return-value]

    async def retrieve(
        self,
        video_id: str,
        *,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> Video:
        response = await self.openai_client.with_raw_response.videos.retrieve(
            video_id=video_id,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        data = Video(**json.loads(response.text))
        data._headers = response.headers

        return data

    async def list(
        self,
        *,
        after: Union[str, Omit] = omit,
        limit: Union[int, Omit] = omit,
        order: Union[Literal["asc", "desc"], Omit] = omit,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> VideoList:
        response = await self.openai_client.with_raw_response.videos.list(
            after=after,
            limit=limit,
            order=order,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        data = VideoList(**json.loads(response.text))
        data._headers = response.headers
        return data

    async def delete(
        self,
        video_id: str,
        *,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> VideoDeleteResponse:
        response = await self.openai_client.with_raw_response.videos.delete(
            video_id=video_id,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        data = VideoDeleteResponse(**json.loads(response.text))
        data._headers = response.headers
        return data

    async def download_content(
        self,
        video_id: str,
        *,
        variant: Union[Literal["video", "thumbnail", "spritesheet"], Omit] = omit,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> Any:
        response = await self.openai_client.videos.download_content(
            video_id=video_id,
            variant=variant,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        return response

    async def remix(
        self,
        video_id: str,
        *,
        prompt: str,
        extra_headers: Optional[Headers] = None,
        extra_query: Optional[Query] = None,
        extra_body: Optional[Body] = None,
        timeout: Union[Optional[float], httpx.Timeout, NotGiven] = not_given,
    ) -> Video:
        response = await self.openai_client.with_raw_response.videos.remix(
            video_id=video_id,
            prompt=prompt,
            extra_headers=extra_headers,
            extra_query=extra_query,
            extra_body=extra_body,
            timeout=timeout,
        )
        data = Video(**json.loads(response.text))
        data._headers = response.headers
        return data
